@@ -84,9 +84,11 @@ namespace Microsoft.Boogie
 
     public bool IsLeftMover => ActionDecl.MoverType == MoverType.Left || ActionDecl.MoverType == MoverType.Both;
 
-    public int PendingAsyncStartIndex => ActionDecl.OutParams.Count;
+    public bool IsConditionalLeftMover => IsLeftMover && ActionDecl.HasPreconditions;
 
-    public Expr ExitCondition => ActionDecl.FindExprAttribute("exit_condition");
+    public bool IsUnconditionalLeftMover => IsLeftMover && !ActionDecl.HasPreconditions;
+
+    public int PendingAsyncStartIndex => ActionDecl.OutParams.Count;
 
     public bool TriviallyCommutesWith(Action other)
     {
@@ -146,6 +148,30 @@ namespace Microsoft.Boogie
       return Substituter.SubstitutionFromDictionary(map);
     }
 
+    public IEnumerable<AssertCmd> Preconditions(int layerNum, Substitution subst)
+    {
+      var errorMessage = $"this precondition of {Name} could not be proved";
+      var cmds = new List<AssertCmd>();
+      ActionDecl.Requires.Where(req => req.Layers.Contains(layerNum)).ForEach(req =>
+      {
+        cmds.Add(CmdHelper.AssertCmd(req.tok, Substituter.Apply(subst, req.Condition), errorMessage));
+      });
+      foreach (var callCmd in ActionDecl.YieldRequires)
+      {
+        var yieldInvariant = (YieldInvariantDecl)callCmd.Proc;
+        if (layerNum == yieldInvariant.Layer)
+        {
+          Substitution callFormalsToActuals = Substituter.SubstitutionFromDictionary(yieldInvariant.InParams
+              .Zip(callCmd.Ins)
+              .ToDictionary(x => x.Item1, x => x.Item2));
+          yieldInvariant.Preserves.ForEach(req =>
+            cmds.Add(CmdHelper.AssertCmd(req.tok,
+                  Substituter.Apply(subst, Substituter.Apply(callFormalsToActuals, req.Condition)), errorMessage)));
+        }
+      }
+      return cmds;
+    }
+
     public static Implementation CreateDuplicateImplementation(Implementation impl, string name)
     {
       var duplicateImpl = new Duplicator().VisitImplementation(impl);
@@ -154,7 +180,7 @@ namespace Microsoft.Boogie
       duplicateImpl.Attributes = null;
       // in case impl.Proc is ActionDecl, convert to Procedure
       duplicateImpl.Proc = new Procedure(proc.tok, name, proc.TypeParameters, proc.InParams,
-        proc.OutParams, proc.IsPure, new List<Requires>(), proc.Modifies, new List<Ensures>());
+        proc.OutParams, proc.IsPure, new List<Requires>(), new List<Requires>(), new List<Ensures>(), proc.Modifies);
       CivlUtil.AddInlineAttribute(duplicateImpl.Proc);
       return duplicateImpl;
     }
@@ -195,7 +221,7 @@ namespace Microsoft.Boogie
 
       var proc = checkerImpl.Proc;
       checkerImpl.Proc = new Procedure(proc.tok, checkerName, proc.TypeParameters, proc.InParams,
-        proc.OutParams, proc.IsPure, requires, proc.Modifies, new List<Ensures>());
+        proc.OutParams, proc.IsPure, requires, new List<Requires>(), new List<Ensures>(), proc.Modifies);
       gateSufficiencyCheckerDecls.AddRange(new Declaration[] { checkerImpl.Proc, checkerImpl });
 
       Wlp.HoistAsserts(Impl, civlTypeChecker.Options);
