@@ -53,8 +53,8 @@ namespace Microsoft.Boogie
 
       SkipActionDecl = new ActionDecl(Token.NoToken, AddNamePrefix("Skip"), MoverType.Both, new List<Variable>(),
         new List<Variable>(), true, new List<ActionDeclRef>(), null, null,
-        new List<Requires>(), new List<CallCmd>(), new List<AssertCmd>(),
-        new List<IdentifierExpr>(), null, null);
+        new List<Requires>(), new List<IdentifierExpr>(), new List<CallCmd>(), new List<AssertCmd>(),
+        null, null);
       var skipImplementation = DeclHelper.Implementation(
         SkipActionDecl,
         new List<Variable>(),
@@ -122,7 +122,7 @@ namespace Microsoft.Boogie
       CreateAtomicActions(actionDecls);
       CreateSequentializations(actionDecls);
       AttributeEraser.Erase(this);
-      YieldSufficiencyTypeChecker.TypeCheck(this);
+      YieldSufficiencyChecker.TypeCheck(this);
     }
 
     private HashSet<ActionDecl> TypeCheckActions()
@@ -441,16 +441,9 @@ namespace Microsoft.Boogie
             {
               checkingContext.Error(formals1[i], $"mismatched type of {inout}-parameter {name1} in {decl2.Name}");
             }
-            else if (checkLinearity &&
-                (QKeyValue.FindStringAttribute(formals1[i].Attributes, CivlAttributes.LINEAR) !=
-                 QKeyValue.FindStringAttribute(formals2[i].Attributes, CivlAttributes.LINEAR) ||
-                 QKeyValue.FindStringAttribute(formals1[i].Attributes, CivlAttributes.LINEAR_IN) !=
-                 QKeyValue.FindStringAttribute(formals2[i].Attributes, CivlAttributes.LINEAR_IN) ||
-                 QKeyValue.FindStringAttribute(formals1[i].Attributes, CivlAttributes.LINEAR_OUT) !=
-                 QKeyValue.FindStringAttribute(formals2[i].Attributes, CivlAttributes.LINEAR_OUT)))
+            else if (checkLinearity && LinearTypeChecker.FindLinearKind(formals1[i]) != LinearTypeChecker.FindLinearKind(formals2[i]))
             {
-              checkingContext.Error(formals1[i],
-                $"mismatched linearity annotation of {inout}-parameter {name1} in {decl2.Name}");
+              checkingContext.Error(formals1[i], $"mismatched linearity annotation of {inout}-parameter {name1} in {decl2.Name}");
             }
           }
         }
