@@ -323,10 +323,10 @@ namespace Microsoft.Boogie
   {
     public static Procedure Procedure(string name,
       List<Variable> inParams, List<Variable> outParams,
-      List<Requires> requires, List<IdentifierExpr> modifies, List<Ensures> ensures,
+      List<Requires> requires, List<Requires> preserves, List<Ensures> ensures, List<IdentifierExpr> modifies,
       QKeyValue kv = null)
     {
-      return new Procedure(Token.NoToken, name, new List<TypeVariable>(), inParams, outParams, false, requires, modifies, ensures, kv);
+      return new Procedure(Token.NoToken, name, new List<TypeVariable>(), inParams, outParams, false, requires, preserves, ensures, modifies, kv);
     }
 
     public static Implementation Implementation(Procedure proc,
