@@ -21,21 +21,18 @@ namespace Microsoft.Boogie
     private AbsyMap absyMap; /* Duplicate -> Original */
     private Dictionary<string, Procedure> asyncCallPreconditionCheckers;
 
-    private Dictionary<CallCmd, CallCmd> refinementCallCmds; // rewritten -> original
-    private Dictionary<CallCmd, Block> refinementBlocks; // rewritten -> block
-
     private LinearRewriter linearRewriter;
 
-    private ConcurrencyOptions Options => civlTypeChecker.Options;
+    private bool doRefinementCheck;
 
-    public YieldingProcDuplicator(CivlTypeChecker civlTypeChecker, int layerNum)
+    public YieldingProcDuplicator(CivlTypeChecker civlTypeChecker, int layerNum, bool doRefinementCheck)
     {
       this.civlTypeChecker = civlTypeChecker;
       this.layerNum = layerNum;
+      this.doRefinementCheck = doRefinementCheck;
       this.procToDuplicate = new Dictionary<Procedure, Procedure>();
       this.absyMap = new AbsyMap();
       this.asyncCallPreconditionCheckers = new Dictionary<string, Procedure>();
-      this.refinementBlocks = new Dictionary<CallCmd, Block>();
       this.linearRewriter = new LinearRewriter(civlTypeChecker);
     }
 
@@ -46,18 +43,36 @@ namespace Microsoft.Boogie
       if (!procToDuplicate.ContainsKey(node))
       {
         Debug.Assert(layerNum <= node.Layer);
+        var procName = civlTypeChecker.AddNamePrefix($"{node.Name}_{layerNum}");
+        if (doRefinementCheck)
+        {
+          procName = civlTypeChecker.AddNamePrefix($"{node.Name}_Refine_{layerNum}");
+        }
+        var requires = VisitRequiresSeq(node.Requires);
+        var preserves = VisitRequiresSeq(node.Preserves);
+        var ensures = VisitEnsuresSeq(node.Ensures);
+        if (node.HasMoverType && layerNum == node.Layer && !doRefinementCheck)
+        {
+          requires = requires.Select(req => new Requires(req.tok, true, req.Condition, req.Comment, req.Attributes)).ToList();
+          preserves = preserves.Select(req => new Requires(req.tok, true, req.Condition, req.Comment, req.Attributes)).ToList();
+          ensures = ensures.Select(ens => new Ensures(ens.tok, true, ens.Condition, ens.Comment, ens.Attributes)).ToList();
+        }
+        requires.AddRange(preserves);
+        ensures.AddRange(preserves.Select(req => new Ensures(req.tok, req.Free, req.Condition, req.Comment, req.Attributes)));
         var proc = new Procedure(
           node.tok,
-          civlTypeChecker.AddNamePrefix($"{node.Name}_{layerNum}"),
+          procName,
           new List<TypeVariable>(),
           VisitVariableSeq(node.InParams),
           VisitVariableSeq(node.OutParams),
           false,
-          VisitRequiresSeq(node.Requires),
+          requires,
+          new List<Requires>(),
+          ensures,
           (node.HasMoverType && node.Layer == layerNum
             ? node.ModifiedVars.Select(g => Expr.Ident(g))
-            : civlTypeChecker.GlobalVariables.Select(v => Expr.Ident(v))).ToList(),
-          VisitEnsuresSeq(node.Ensures));
+            : civlTypeChecker.GlobalVariables.Select(v => Expr.Ident(v))).ToList()
+          );
         procToDuplicate[node] = proc;
         absyMap[proc] = node;
       }
@@ -116,7 +131,8 @@ namespace Microsoft.Boogie
 
     private Implementation enclosingImpl;
     private YieldProcedureDecl enclosingYieldingProc;
-    private bool IsRefinementLayer => layerNum == enclosingYieldingProc.Layer;
+    private Block enclosingBlock;
+    private bool inPredicatePhase;
 
     private Action RefinedAction => civlTypeChecker.Action(enclosingYieldingProc.RefinedAction.ActionDecl);
 
@@ -153,29 +169,12 @@ namespace Microsoft.Boogie
 
       returnedPAs = new Dictionary<CtorType, Variable>();
 
-      refinementCallCmds = new Dictionary<CallCmd, CallCmd>();
       Implementation newImpl = base.VisitImplementation(impl);
       newImpl.Name = newImpl.Proc.Name;
-
-      foreach (var kv in refinementCallCmds)
-      {
-        var rewrittenCallCmd = kv.Key;
-        var originalCallCmd = kv.Value;
-        newCmdSeq = new List<Cmd>();
-        AddActionCall(originalCallCmd, (YieldProcedureDecl)originalCallCmd.Proc);
-        var block = BlockHelper.Block(
-          civlTypeChecker.AddNamePrefix($"call_refinement_{refinementBlocks.Count}"),
-          newCmdSeq,
-          new List<Block>());
-        refinementBlocks.Add(rewrittenCallCmd, block);
-        newCmdSeq = null;
-      }
-
-      refinementCallCmds = null;
       
       newImpl.LocVars.AddRange(returnedPAs.Values);
 
-      if (!enclosingYieldingProc.HasMoverType && RefinedAction.HasPendingAsyncs && IsRefinementLayer)
+      if (!enclosingYieldingProc.HasMoverType && RefinedAction.HasPendingAsyncs && doRefinementCheck)
       {
         var assumeExpr = EmptyPendingAsyncMultisetExpr(CollectedPAs, RefinedAction.PendingAsyncs);
         newImpl.LocVars.AddRange(civlTypeChecker.PendingAsyncCollectors(impl).Values.Except(impl.LocVars));
@@ -188,19 +187,34 @@ namespace Microsoft.Boogie
 
     public override Block VisitBlock(Block node)
     {
+      enclosingBlock = node;
       var block = base.VisitBlock(node);
+      enclosingBlock = null;
       absyMap[block] = node;
       return block;
     }
 
     public override Cmd VisitAssertCmd(AssertCmd node)
     {
-      var assertCmd = (AssertCmd) base.VisitAssertCmd(node);
+      var assertCmd = (AssertCmd)base.VisitAssertCmd(node);
       if (!node.Layers.Contains(layerNum))
       {
         assertCmd.Expr = Expr.True;
+        return assertCmd;
+      }
+      if (layerNum != enclosingYieldingProc.Layer)
+      {
+        return assertCmd;
+      }
+      bool insideLoopInvariantOfYieldingLoop = enclosingYieldingProc.IsYieldingLoopHeaderAtProcedureLayer(enclosingBlock) && inPredicatePhase;
+      if (insideLoopInvariantOfYieldingLoop)
+      {
+        return doRefinementCheck ? new AssumeCmd(node.tok, assertCmd.Expr, node.Attributes) : assertCmd;
+      }
+      else
+      {
+        return doRefinementCheck ? assertCmd : new AssumeCmd(node.tok, assertCmd.Expr, node.Attributes);
       }
-      return assertCmd;
     }
 
     public override Cmd VisitCallCmd(CallCmd call)
@@ -218,17 +232,16 @@ namespace Microsoft.Boogie
       {
         absyMap[newCall] = parCall;
       }
-
       return newParCall;
     }
 
     public override List<Cmd> VisitCmdSeq(List<Cmd> cmdSeq)
     {
       newCmdSeq = new List<Cmd>();
+      inPredicatePhase = true;
       foreach (var cmd in cmdSeq)
       {
         Cmd newCmd = (Cmd) Visit(cmd);
-
         if (newCmd is CallCmd)
         {
           ProcessCallCmd((CallCmd) newCmd);
@@ -241,6 +254,7 @@ namespace Microsoft.Boogie
         {
           newCmdSeq.Add(newCmd);
         }
+        inPredicatePhase = inPredicatePhase && (cmd is PredicateCmd || cmd is CallCmd callCmd && callCmd.Proc is YieldInvariantDecl);
       }
       return newCmdSeq;
     }
@@ -252,16 +266,22 @@ namespace Microsoft.Boogie
         var callLayerRange = newCall.LayerRange;
         if (callLayerRange.Contains(layerNum))
         {
+          bool makeAssume = layerNum == enclosingYieldingProc.Layer && !doRefinementCheck;
           if (newCall.Proc is ActionDecl actionDecl)
           {
             var pureAction = civlTypeChecker.Action(actionDecl);
             newCall.Proc = pureAction.Impl.Proc;
-            InjectGate(pureAction, newCall);
+            InjectGate(pureAction, newCall, makeAssume);
             newCmdSeq.Add(newCall);
           }
           else if (CivlPrimitives.IsPrimitive(newCall.Proc))
           {
-            newCmdSeq.AddRange(linearRewriter.RewriteCallCmd(newCall));
+            var cmds = linearRewriter.RewriteCallCmd(newCall).Select(cmd =>
+              cmd is AssertCmd assertCmd && makeAssume
+              ? new AssumeCmd(assertCmd.tok, assertCmd.Expr, assertCmd.Attributes)
+              : cmd
+            );
+            newCmdSeq.AddRange(cmds);
           }
           else
           {
@@ -295,7 +315,7 @@ namespace Microsoft.Boogie
             // synchronize the called atomic action
             AddActionCall(newCall, yieldingProc);
           }
-          else if (IsRefinementLayer)
+          else if (doRefinementCheck)
           {
             AddPendingAsync(newCall, yieldingProc);
           }
@@ -307,14 +327,14 @@ namespace Microsoft.Boogie
             // synchronize the called mover procedure
             AddDuplicateCall(newCall, false);
           }
+          else if (doRefinementCheck)
+          {
+            Debug.Assert(!yieldingProc.HasMoverType);
+            AddPendingAsync(newCall, yieldingProc);
+          }
           else
           {
             DesugarAsyncCall(newCall);
-            if (IsRefinementLayer)
-            {
-              Debug.Assert(!yieldingProc.HasMoverType);
-              AddPendingAsync(newCall, yieldingProc);
-            }
           }
         }
         return;
@@ -325,7 +345,7 @@ namespace Microsoft.Boogie
       {
         AddDuplicateCall(newCall, yieldingProc.Layer > layerNum);
       }
-      else if (!yieldingProc.HasMoverType)
+      else
       {
         if (yieldingProc.Layer < layerNum)
         {
@@ -333,57 +353,71 @@ namespace Microsoft.Boogie
         }
         else
         {
-          if (IsRefinementLayer && layerNum == yieldingProc.Layer && yieldingProc.RefinedAction != null)
+          if (doRefinementCheck && yieldingProc.RefinedAction.ActionDecl != civlTypeChecker.SkipActionDecl)
           {
-            refinementCallCmds[newCall] = (CallCmd) VisitCallCmd(newCall);
+            var parCallCmdBefore = new ParCallCmd(newCall.tok, []);
+            absyMap[parCallCmdBefore] = absyMap[newCall];
+            newCmdSeq.Add(parCallCmdBefore);
+            AddActionCall(newCall, yieldingProc);
+            var parCallCmdAfter = new ParCallCmd(newCall.tok, []);
+            absyMap[parCallCmdAfter] = absyMap[newCall];
+            newCmdSeq.Add(parCallCmdAfter);
+          }
+          else
+          {
+            AddDuplicateCall(newCall, true);
           }
-
-          AddDuplicateCall(newCall, true);
         }
-
         Debug.Assert(newCall.Outs.Count == newCall.Proc.OutParams.Count);
       }
-      else
-      {
-        Debug.Assert(false);
-      }
     }
 
     private void ProcessParCallCmd(ParCallCmd newParCall)
     {
       var callCmds = new List<CallCmd>();
+
+      void AddParallelCall()
+      {
+        var parCallCmd = new ParCallCmd(newParCall.tok, callCmds);
+        absyMap[parCallCmd] = absyMap[newParCall];
+        newCmdSeq.Add(parCallCmd);
+      }
+
+      void ProcessPendingCallCmds()
+      {
+        if (callCmds.Count == 0)
+        {
+          return;
+        }
+        var callCmd = callCmds.Find(cmd =>
+            cmd.Proc is YieldProcedureDecl yieldingProc &&
+            layerNum == yieldingProc.Layer &&
+            yieldingProc.RefinedAction.ActionDecl != civlTypeChecker.SkipActionDecl);
+        if (doRefinementCheck && callCmd != null)
+        {
+          callCmds.Remove(callCmd);
+          callCmds.ForEach(callCmd => Debug.Assert(callCmd.Proc is YieldInvariantDecl));
+          AddParallelCall();
+          AddActionCall(callCmd, (YieldProcedureDecl)callCmd.Proc);
+        }
+        AddParallelCall();
+        callCmds = [];
+      }
+
       foreach (var callCmd in newParCall.CallCmds)
       {
-        if (callCmd.Proc is YieldProcedureDecl)
+        if (callCmd.Proc is YieldProcedureDecl yieldingProc)
         {
-          var yieldingProc = (YieldProcedureDecl)callCmd.Proc;
-          if (layerNum > yieldingProc.Layer && !yieldingProc.HasMoverType ||
-              layerNum == yieldingProc.Layer && yieldingProc.HasMoverType)
+          Debug.Assert(layerNum <= yieldingProc.Layer || !yieldingProc.HasMoverType);
+          if (layerNum > yieldingProc.Layer || layerNum == yieldingProc.Layer && yieldingProc.HasMoverType)
           {
-            if (callCmds.Count > 0)
-            {
-              var parCallCmd = new ParCallCmd(newParCall.tok, callCmds);
-              absyMap[parCallCmd] = absyMap[newParCall];
-              newCmdSeq.Add(parCallCmd);
-              callCmds = new List<CallCmd>();
-            }
-
+            ProcessPendingCallCmds();
             ProcessCallCmd(callCmd);
             continue;
           }
         }
-
         if (procToDuplicate.ContainsKey(callCmd.Proc))
         {
-          var yieldingProc = (YieldProcedureDecl)callCmd.Proc;
-          if (!yieldingProc.HasMoverType)
-          {
-            if (IsRefinementLayer && layerNum == yieldingProc.Layer && yieldingProc.RefinedAction != null)
-            {
-              refinementCallCmds[callCmd] = (CallCmd) VisitCallCmd(callCmd);
-            }
-          }
-
           callCmd.Proc = procToDuplicate[callCmd.Proc];
           callCmd.callee = callCmd.Proc.Name;
           callCmds.Add(callCmd);
@@ -398,12 +432,7 @@ namespace Microsoft.Boogie
         }
       }
 
-      if (callCmds.Count > 0)
-      {
-        var parCallCmd = new ParCallCmd(newParCall.tok, callCmds);
-        absyMap[parCallCmd] = absyMap[newParCall];
-        newCmdSeq.Add(parCallCmd);
-      }
+      ProcessPendingCallCmds();
     }
 
     private Action PrepareNewCall(CallCmd newCall, YieldProcedureDecl calleeActionProc)
@@ -444,7 +473,8 @@ namespace Microsoft.Boogie
     private void AddActionCall(CallCmd newCall, YieldProcedureDecl calleeActionProc)
     {
       var calleeRefinedAction = PrepareNewCall(newCall, calleeActionProc);
-      InjectGate(calleeRefinedAction, newCall, !IsRefinementLayer);
+      InjectPreconditions(calleeRefinedAction, newCall);
+      InjectGate(calleeRefinedAction, newCall, !doRefinementCheck);
       newCmdSeq.Add(newCall);
 
       if (calleeRefinedAction.HasPendingAsyncs)
@@ -454,7 +484,22 @@ namespace Microsoft.Boogie
       }
     }
 
-    private void InjectGate(Action action, CallCmd callCmd, bool assume = false)
+    private void InjectPreconditions(Action action, CallCmd callCmd)
+    {
+      if (!action.ActionDecl.HasPreconditions)
+      {
+        return;
+      }
+      Dictionary<Variable, Expr> map = new Dictionary<Variable, Expr>();
+      for (int i = 0; i < action.ActionDecl.InParams.Count; i++)
+      {
+        map[action.ActionDecl.InParams[i]] = callCmd.Ins[i];
+      }
+      Substitution subst = Substituter.SubstitutionFromDictionary(map);
+      newCmdSeq.AddRange(action.Preconditions(layerNum, subst));
+    }
+
+    private void InjectGate(Action action, CallCmd callCmd, bool assume)
     {
       if (action.Gate.Count == 0)
       {
@@ -494,7 +539,7 @@ namespace Microsoft.Boogie
     {
       // Inject pending async collection
       newCall.Outs.AddRange(calleeRefinedAction.PendingAsyncs.Select(decl => Expr.Ident(ReturnedPAs(decl.PendingAsyncType))));
-      if (!IsRefinementLayer)
+      if (!doRefinementCheck)
       {
         return;
       }
@@ -549,7 +594,7 @@ namespace Microsoft.Boogie
         asyncCallPreconditionCheckers[newCall.Proc.Name] = DeclHelper.Procedure(
           civlTypeChecker.AddNamePrefix($"AsyncCall_{newCall.Proc.Name}_{layerNum}"),
           newCall.Proc.InParams, newCall.Proc.OutParams,
-          procToDuplicate[newCall.Proc].Requires, new List<IdentifierExpr>(), new List<Ensures>());
+          procToDuplicate[newCall.Proc].Requires, new List<Requires>(), new List<Ensures>(), new List<IdentifierExpr>());
       }
 
       var asyncCallPreconditionChecker = asyncCallPreconditionCheckers[newCall.Proc.Name];
@@ -605,7 +650,7 @@ namespace Microsoft.Boogie
         civlTypeChecker,
         layerNum,
         absyMap,
-        refinementBlocks));
+        doRefinementCheck));
       return decls;
     }
   }
