@@ -283,20 +283,6 @@ public class CallCmd : CallCommonality
       return;
     }
 
-    var callerModifiedVars = new HashSet<Variable>(callerDecl.ModifiedVars);
-
-    void CheckModifies(IEnumerable<Variable> modifiedVars)
-    {
-      if (!callerDecl.HasMoverType)
-      {
-        return;
-      }
-      foreach (var v in modifiedVars.Where(v => !callerModifiedVars.Contains(v)))
-      {
-        tc.Error(this, $"modified variable does not appear in modifies clause of mover procedure: {v.Name}");
-      }
-    }
-
     // check layers
     if (Proc is YieldProcedureDecl calleeDecl)
     {
@@ -318,7 +304,6 @@ public class CallCmd : CallCommonality
             }
             else
             {
-              CheckModifies(highestRefinedActionDecl.ModifiedVars);
               if (callerDecl.HasMoverType && highestRefinedActionDecl.Creates.Any())
               {
                 tc.Error(this, "caller must not be a mover procedure");
@@ -388,7 +373,6 @@ public class CallCmd : CallCommonality
         }
         else
         {
-          CheckModifies(calleeDecl.ModifiedVars);
           if (IsAsync)
           {
             if (!isSynchronized)
@@ -409,6 +393,10 @@ public class CallCmd : CallCommonality
       {
         tc.Error(this, "layer of callee must not be more than layer of caller");
       }
+      else if (yieldInvariantDecl.Layer == callerDecl.Layer && callerDecl.HasMoverType)
+      {
+        tc.Error(this, "layer of callee must be less than layer of caller");
+      }
     }
     else
     {
@@ -602,19 +590,21 @@ public class CallCmd : CallCommonality
     }
 
     // typecheck in-parameters
+    var oldGlobalAccessOnlyInOld = tc.GlobalAccessOnlyInOld;
+    tc.GlobalAccessOnlyInOld = oldGlobalAccessOnlyInOld || tc.Proc is YieldProcedureDecl && Proc is YieldProcedureDecl;
     for (int i = 0; i < Ins.Count; i++)
     {
       var e = Ins[i];
       if (e != null)
       {
-        tc.GlobalAccessOnlyInOld = tc.Proc is YieldProcedureDecl && Proc is YieldProcedureDecl;
         tc.ExpectedLayerRange = expectedLayerRanges?[i];
         e.Typecheck(tc);
-        tc.GlobalAccessOnlyInOld = false;
         tc.ExpectedLayerRange = null;
       }
     }
+    tc.GlobalAccessOnlyInOld = oldGlobalAccessOnlyInOld;
 
+    // typecheck out-parameters
     for (int i = 0; i < Outs.Count; i++)
     {
       var e = Outs[i];
