@@ -6,7 +6,7 @@ public class ReturnCmd : TransferCmd
 {
   public QKeyValue Attributes { get; set; }
     
-  public ReturnCmd(IToken /*!*/ tok)
+  public ReturnCmd(IToken tok)
     : base(tok)
   {
     Contract.Requires(tok != null);
@@ -14,19 +14,16 @@ public class ReturnCmd : TransferCmd
 
   public override void Emit(TokenTextWriter stream, int level)
   {
-    //Contract.Requires(stream != null);
     stream.WriteLine(this, level, "return;");
   }
 
   public override void Resolve(ResolutionContext rc)
   {
-    //Contract.Requires(rc != null);
     // nothing to resolve
   }
 
   public override Absy StdDispatch(StandardVisitor visitor)
   {
-    //Contract.Requires(visitor != null);
     Contract.Ensures(Contract.Result<Absy>() != null);
     return visitor.VisitReturnCmd(this);
   }
