@@ -1744,13 +1744,11 @@ namespace Microsoft.Boogie
      {:sync}
        Synchronized async call.
 
-     {:linear ""domain""}
-       Permission type for domain.
-       Collector function for domain.
-       Linear variable (both global and local).
+     {:linear}
+       Linear variable (both global and input/output parameter).
 
-     {:linear_in ""domain""}
-     {:linear_out ""domain""}
+     {:linear_in}
+     {:linear_out}
        Linear input/output parameter.";
 
     protected override string HelpHeader =>
@@ -1811,8 +1809,8 @@ namespace Microsoft.Boogie
                 unroll loops, following up to n back edges (and then some)
                 default is -1, which means loops are not unrolled
   /extractLoops
-                extract reducible loops into recursive procedures and
-                inline irreducible loops using the bound supplied by /loopUnroll:<n>
+                convert all irreducible loops to reducible forms by node splitting
+                and extract all loops into recursive procedures
   /soundLoopUnrolling
                 sound loop unrolling
   /kInductionDepth:<k>
