@@ -23,15 +23,15 @@ namespace Microsoft.Boogie.TypeErasure
 
   internal struct UntypedFunction
   {
-    public readonly Function /*!*/
+    public readonly Function
       Fun;
 
     // type parameters that can be extracted from the value parameters
-    public readonly List<TypeVariable /*!*/> /*!*/
+    public readonly List<TypeVariable>
       ImplicitTypeParams;
 
     // type parameters that have to be given explicitly
-    public readonly List<TypeVariable /*!*/> /*!*/
+    public readonly List<TypeVariable>
       ExplicitTypeParams;
 
     [ContractInvariantMethod]
@@ -43,9 +43,9 @@ namespace Microsoft.Boogie.TypeErasure
     }
 
 
-    public UntypedFunction(Function /*!*/ fun,
-      List<TypeVariable /*!*/> /*!*/ implicitTypeParams,
-      List<TypeVariable /*!*/> /*!*/ explicitTypeParams)
+    public UntypedFunction(Function fun,
+      List<TypeVariable> implicitTypeParams,
+      List<TypeVariable> explicitTypeParams)
     {
       Contract.Requires(fun != null);
       Contract.Requires(Cce.NonNullElements(implicitTypeParams));
@@ -71,7 +71,7 @@ namespace Microsoft.Boogie.TypeErasure
       this.Options = options;
 
       TypeFunction = HelperFuns.BoogieFunction("dummy", Type.Int);
-      Typed2UntypedFunctions = new Dictionary<Function /*!*/, UntypedFunction>();
+      Typed2UntypedFunctions = new Dictionary<Function, UntypedFunction>();
       MapTypeAbstracterAttr = null;
     }
 
@@ -84,7 +84,7 @@ namespace Microsoft.Boogie.TypeErasure
       this.Options = builder.Options;
       TypeFunction = builder.TypeFunction;
       Typed2UntypedFunctions =
-        new Dictionary<Function /*!*/, UntypedFunction>(builder.Typed2UntypedFunctions);
+        new Dictionary<Function, UntypedFunction>(builder.Typed2UntypedFunctions);
 
       MapTypeAbstracterAttr =
         builder.MapTypeAbstracterAttr == null
@@ -110,20 +110,18 @@ namespace Microsoft.Boogie.TypeErasure
     //                                          type(x)=int ==> Int2U(U2Int(x))==x"
     protected override VCExpr GenReverseCastAxiom(Function castToU, Function castFromU)
     {
-      //Contract.Requires(castFromU != null);
-      //Contract.Requires(castToU != null);
       Contract.Ensures(Contract.Result<VCExpr>() != null);
-      VCExpr /*!*/
+      VCExpr
         eq = GenReverseCastEq(castToU, castFromU, out var var, out var triggers);
       Contract.Assert(Cce.NonNullElements(triggers));
       Contract.Assert(var != null);
       Contract.Assert(eq != null);
-      VCExpr /*!*/
+      VCExpr
         premiss;
       premiss = GenVarTypeAxiom(var, Cce.NonNull(castFromU.OutParams[0]).TypedIdent.Type,
         // we don't have any bindings available
-        new Dictionary<TypeVariable /*!*/, VCExpr /*!*/>());
-      VCExpr /*!*/
+        new Dictionary<TypeVariable, VCExpr>());
+      VCExpr
         matrix = Gen.ImpliesSimp(premiss, eq);
       Contract.Assert(matrix != null);
       return Gen.Forall(HelperFuns.ToList(var), triggers, "cast:" + castFromU.Name, 1, matrix);
@@ -131,18 +129,16 @@ namespace Microsoft.Boogie.TypeErasure
 
     protected override VCExpr GenCastTypeAxioms(Function castToU, Function castFromU)
     {
-      //Contract.Requires(castFromU != null);
-      //Contract.Requires(castToU != null);
       Contract.Ensures(Contract.Result<VCExpr>() != null);
-      Type /*!*/
+      Type
         fromType = Cce.NonNull(castToU.InParams[0]).TypedIdent.Type;
-      return GenFunctionAxiom(castToU, new List<TypeVariable /*!*/>(), new List<TypeVariable /*!*/>(),
+      return GenFunctionAxiom(castToU, new List<TypeVariable>(), new List<TypeVariable>(),
         HelperFuns.ToList(fromType), fromType);
     }
 
     private MapTypeAbstractionBuilderPremisses MapTypeAbstracterAttr;
 
-    internal override MapTypeAbstractionBuilder /*!*/ MapTypeAbstracter
+    internal override MapTypeAbstractionBuilder MapTypeAbstracter
     {
       get
       {
@@ -157,7 +153,7 @@ namespace Microsoft.Boogie.TypeErasure
       }
     }
 
-    internal MapTypeAbstractionBuilderPremisses /*!*/ MapTypeAbstracterPremisses
+    internal MapTypeAbstractionBuilderPremisses MapTypeAbstracterPremisses
     {
       get
       {
@@ -171,7 +167,7 @@ namespace Microsoft.Boogie.TypeErasure
 
     // function that maps individuals to their type
     // the field is overwritten with its actual value in "Setup"
-    private Function /*!*/
+    private Function
       TypeFunction;
 
     [ContractInvariantMethod]
@@ -192,12 +188,12 @@ namespace Microsoft.Boogie.TypeErasure
     // Generate type premisses and type parameter bindings for quantifiers, functions, procedures
 
     // let-bindings to extract the instantiations of type parameters
-    public List<VCExprLetBinding /*!*/> /*!*/
+    public List<VCExprLetBinding>
       GenTypeParamBindings( // the original bound variables and (implicit) type parameters
-        List<TypeVariable /*!*/> /*!*/ typeParams, List<VCExprVar /*!*/> /*!*/ oldBoundVars,
+        List<TypeVariable> typeParams, List<VCExprVar> oldBoundVars,
         // VariableBindings to which the translation
         // TypeVariable -> VCExprVar is added
-        VariableBindings /*!*/ bindings,
+        VariableBindings bindings,
         bool addTypeVarsToBindings)
     {
       Contract.Requires(typeParams != null);
@@ -210,7 +206,7 @@ namespace Microsoft.Boogie.TypeErasure
       // let-expression
       if (addTypeVarsToBindings)
       {
-        foreach (TypeVariable /*!*/ tvar in typeParams)
+        foreach (TypeVariable tvar in typeParams)
         {
           Contract.Assert(tvar != null);
           bindings.TypeVariableBindings.Add(tvar, Gen.Variable(tvar.Name, T));
@@ -218,13 +214,13 @@ namespace Microsoft.Boogie.TypeErasure
       }
 
       // extract the values of type variables from the term variables
-      List<VCExprVar /*!*/> /*!*/
-        UtypedVars = new List<VCExprVar /*!*/>(oldBoundVars.Count);
-      List<Type /*!*/> /*!*/
-        originalTypes = new List<Type /*!*/>(oldBoundVars.Count);
+      List<VCExprVar>
+        UtypedVars = new List<VCExprVar>(oldBoundVars.Count);
+      List<Type>
+        originalTypes = new List<Type>(oldBoundVars.Count);
       foreach (VCExprVar var in oldBoundVars)
       {
-        VCExprVar /*!*/
+        VCExprVar
           newVar = bindings.VCExprVarBindings[var];
         if (newVar.Type.Equals(U))
         {
@@ -240,16 +236,16 @@ namespace Microsoft.Boogie.TypeErasure
     }
 
 
-    public VCExpr /*!*/ AddTypePremisses(List<VCExprLetBinding /*!*/> /*!*/ typeVarBindings,
-      VCExpr /*!*/ typePremisses, bool universal,
-      VCExpr /*!*/ body)
+    public VCExpr AddTypePremisses(List<VCExprLetBinding> typeVarBindings,
+      VCExpr typePremisses, bool universal,
+      VCExpr body)
     {
       Contract.Requires(Cce.NonNullElements(typeVarBindings));
       Contract.Requires(typePremisses != null);
       Contract.Requires(body != null);
       Contract.Ensures(Contract.Result<VCExpr>() != null);
 
-      VCExpr /*!*/
+      VCExpr
         bodyWithPremisses;
       if (universal)
       {
@@ -269,10 +265,10 @@ namespace Microsoft.Boogie.TypeErasure
     // term variables. E.g., for a function  f<a>(x : C a), we would extract the
     // instantiation of "a" by looking at the concrete type of "x".
 
-    public List<VCExprLetBinding /*!*/> /*!*/
-      BestTypeVarExtractors(List<TypeVariable /*!*/> /*!*/ vars, List<Type /*!*/> /*!*/ types,
-        List<VCExprVar /*!*/> /*!*/ concreteTypeSources,
-        VariableBindings /*!*/ bindings)
+    public List<VCExprLetBinding>
+      BestTypeVarExtractors(List<TypeVariable> vars, List<Type> types,
+        List<VCExprVar> concreteTypeSources,
+        VariableBindings bindings)
     {
       Contract.Requires(Cce.NonNullElements(vars));
       Contract.Requires(Cce.NonNullElements(types));
@@ -280,9 +276,9 @@ namespace Microsoft.Boogie.TypeErasure
       Contract.Requires(bindings != null);
       Contract.Ensures(Cce.NonNullElements(Contract.Result<List<VCExprLetBinding>>()));
 
-      List<VCExprLetBinding /*!*/> /*!*/
-        typeParamBindings = new List<VCExprLetBinding /*!*/>();
-      foreach (TypeVariable /*!*/ var in vars)
+      List<VCExprLetBinding>
+        typeParamBindings = new List<VCExprLetBinding>();
+      foreach (TypeVariable var in vars)
       {
         Contract.Assert(var != null);
         VCExpr extractor = BestTypeVarExtractor(var, types, concreteTypeSources);
@@ -297,13 +293,13 @@ namespace Microsoft.Boogie.TypeErasure
       return typeParamBindings;
     }
 
-    private VCExpr BestTypeVarExtractor(TypeVariable /*!*/ var, List<Type /*!*/> /*!*/ types,
-      List<VCExprVar /*!*/> /*!*/ concreteTypeSources)
+    private VCExpr BestTypeVarExtractor(TypeVariable var, List<Type> types,
+      List<VCExprVar> concreteTypeSources)
     {
       Contract.Requires(var != null);
       Contract.Requires(Cce.NonNullElements(types));
       Contract.Requires(Cce.NonNullElements(concreteTypeSources));
-      List<VCExpr /*!*/> allExtractors = TypeVarExtractors(var, types, concreteTypeSources);
+      List<VCExpr> allExtractors = TypeVarExtractors(var, types, concreteTypeSources);
       Contract.Assert(Cce.NonNullElements(allExtractors));
       if (allExtractors.Count == 0)
       {
@@ -325,16 +321,16 @@ namespace Microsoft.Boogie.TypeErasure
       return bestExtractor;
     }
 
-    private List<VCExpr /*!*/> /*!*/ TypeVarExtractors(TypeVariable /*!*/ var, List<Type /*!*/> /*!*/ types,
-      List<VCExprVar /*!*/> /*!*/ concreteTypeSources)
+    private List<VCExpr> TypeVarExtractors(TypeVariable var, List<Type> types,
+      List<VCExprVar> concreteTypeSources)
     {
       Contract.Requires(var != null);
       Contract.Requires(Cce.NonNullElements(types));
       Contract.Requires(Cce.NonNullElements(concreteTypeSources));
       Contract.Requires((types.Count == concreteTypeSources.Count));
       Contract.Ensures(Cce.NonNullElements(Contract.Result<List<VCExpr>>()));
-      List<VCExpr /*!*/> /*!*/
-        res = new List<VCExpr /*!*/>();
+      List<VCExpr>
+        res = new List<VCExpr>();
       for (int i = 0; i < types.Count; ++i)
       {
         TypeVarExtractors(var, types[i], TypeOf(concreteTypeSources[i]), res);
@@ -344,7 +340,7 @@ namespace Microsoft.Boogie.TypeErasure
     }
 
     private void TypeVarExtractors(TypeVariable var, Type completeType, VCExpr innerTerm,
-      List<VCExpr /*!*/> /*!*/ extractors)
+      List<VCExpr> extractors)
     {
       Contract.Requires(innerTerm != null);
       Contract.Requires(completeType != null);
@@ -363,7 +359,7 @@ namespace Microsoft.Boogie.TypeErasure
       }
       else if (completeType.IsCtor)
       {
-        CtorType /*!*/
+        CtorType
           ctorType = completeType.AsCtor;
         if (ctorType.Arguments.Count > 0)
         {
@@ -372,7 +368,7 @@ namespace Microsoft.Boogie.TypeErasure
           TypeCtorRepr repr = GetTypeCtorReprStruct(ctorType.Decl);
           for (int i = 0; i < ctorType.Arguments.Count; ++i)
           {
-            VCExpr /*!*/
+            VCExpr
               newInnerTerm = Gen.Function(repr.Dtors[i], innerTerm);
             Contract.Assert(newInnerTerm != null);
             TypeVarExtractors(var, ctorType.Arguments[i], newInnerTerm, extractors);
@@ -394,7 +390,7 @@ namespace Microsoft.Boogie.TypeErasure
     // Symbols for representing functions
 
     // Globally defined functions
-    private readonly IDictionary<Function /*!*/, UntypedFunction /*!*/> /*!*/
+    private readonly IDictionary<Function, UntypedFunction>
       Typed2UntypedFunctions;
 
     [ContractInvariantMethod]
@@ -404,27 +400,27 @@ namespace Microsoft.Boogie.TypeErasure
     }
 
     // distinguish between implicit and explicit type parameters
-    internal static void SeparateTypeParams(List<Type /*!*/> /*!*/ valueArgumentTypes,
-      List<TypeVariable> /*!*/ allTypeParams,
-      out List<TypeVariable /*!*/> /*!*/ implicitParams,
-      out List<TypeVariable /*!*/> /*!*/ explicitParams)
+    internal static void SeparateTypeParams(List<Type> valueArgumentTypes,
+      List<TypeVariable> allTypeParams,
+      out List<TypeVariable> implicitParams,
+      out List<TypeVariable> explicitParams)
     {
       Contract.Requires(Cce.NonNullElements(valueArgumentTypes));
       Contract.Requires(allTypeParams != null);
       Contract.Ensures(Cce.NonNullElements(Contract.ValueAtReturn(out implicitParams)));
       Contract.Ensures(Cce.NonNullElements(Contract.ValueAtReturn(out explicitParams)));
-      List<TypeVariable> /*!*/
+      List<TypeVariable>
         varsInInParamTypes = new List<TypeVariable>();
-      foreach (Type /*!*/ t in valueArgumentTypes)
+      foreach (Type t in valueArgumentTypes)
       {
         Contract.Assert(t != null);
         varsInInParamTypes.AppendWithoutDups(t.FreeVariables);
       }
 
-      implicitParams = new List<TypeVariable /*!*/>(allTypeParams.Count);
-      explicitParams = new List<TypeVariable /*!*/>(allTypeParams.Count);
+      implicitParams = new List<TypeVariable>(allTypeParams.Count);
+      explicitParams = new List<TypeVariable>(allTypeParams.Count);
 
-      foreach (TypeVariable /*!*/ var in allTypeParams)
+      foreach (TypeVariable var in allTypeParams)
       {
         Contract.Assert(var != null);
         if (varsInInParamTypes.Contains(var))
@@ -454,13 +450,13 @@ namespace Microsoft.Boogie.TypeErasure
             UnchangedType(Cce.NonNull(fun.OutParams[0]).TypedIdent.Type) &&
             fun.TypeParameters.Count == 0)
         {
-          res = new UntypedFunction(fun, new List<TypeVariable /*!*/>(), new List<TypeVariable /*!*/>());
+          res = new UntypedFunction(fun, new List<TypeVariable>(), new List<TypeVariable>());
         }
         else
         {
-          List<Type /*!*/> /*!*/
-            argTypes = new List<Type /*!*/>();
-          foreach (Variable /*!*/ v in fun.InParams)
+          List<Type>
+            argTypes = new List<Type>();
+          foreach (Variable v in fun.InParams)
           {
             Contract.Assert(v != null);
             argTypes.Add(v.TypedIdent.Type);
@@ -468,7 +464,7 @@ namespace Microsoft.Boogie.TypeErasure
 
           SeparateTypeParams(argTypes, fun.TypeParameters, out var implicitParams, out var explicitParams);
 
-          Type[] /*!*/
+          Type[]
             types = new Type[explicitParams.Count + fun.InParams.Count + 1];
           int i = 0;
           for (int j = 0; j < explicitParams.Count; ++j)
@@ -484,7 +480,7 @@ namespace Microsoft.Boogie.TypeErasure
 
           types[types.Length - 1] = TypeAfterErasure(Cce.NonNull(fun.OutParams[0]).TypedIdent.Type);
 
-          Function /*!*/
+          Function
             untypedFun = HelperFuns.BoogieFunction(fun.Name, types);
           Contract.Assert(untypedFun != null);
           untypedFun.Attributes = fun.Attributes;
@@ -505,9 +501,9 @@ namespace Microsoft.Boogie.TypeErasure
     {
       Contract.Requires(originalFun != null);
       Contract.Ensures(Contract.Result<VCExpr>() != null);
-      List<Type /*!*/> /*!*/
-        originalInTypes = new List<Type /*!*/>(originalFun.InParams.Count);
-      foreach (Formal /*!*/ f in originalFun.InParams)
+      List<Type>
+        originalInTypes = new List<Type>(originalFun.InParams.Count);
+      foreach (Formal f in originalFun.InParams)
       {
         originalInTypes.Add(f.TypedIdent.Type);
       }
@@ -517,11 +513,11 @@ namespace Microsoft.Boogie.TypeErasure
         Cce.NonNull(originalFun.OutParams[0]).TypedIdent.Type);
     }
 
-    internal VCExpr /*!*/ GenFunctionAxiom(Function /*!*/ fun,
-      List<TypeVariable /*!*/> /*!*/ implicitTypeParams,
-      List<TypeVariable /*!*/> /*!*/ explicitTypeParams,
-      List<Type /*!*/> /*!*/ originalInTypes,
-      Type /*!*/ originalResultType)
+    internal VCExpr GenFunctionAxiom(Function fun,
+      List<TypeVariable> implicitTypeParams,
+      List<TypeVariable> explicitTypeParams,
+      List<Type> originalInTypes,
+      Type originalResultType)
     {
       Contract.Requires(Cce.NonNullElements(implicitTypeParams));
       Contract.Requires(fun != null);
@@ -531,27 +527,27 @@ namespace Microsoft.Boogie.TypeErasure
       Contract.Requires(originalInTypes.Count + explicitTypeParams.Count == fun.InParams.Count);
       Contract.Ensures(Contract.Result<VCExpr>() != null);
 
-      List<VCExprVar /*!*/> /*!*/
-        typedInputVars = new List<VCExprVar /*!*/>(originalInTypes.Count);
+      List<VCExprVar>
+        typedInputVars = new List<VCExprVar>(originalInTypes.Count);
       int i = 0;
-      foreach (Type /*!*/ t in originalInTypes)
+      foreach (Type t in originalInTypes)
       {
         Contract.Assert(t != null);
         typedInputVars.Add(Gen.Variable("arg" + i, t));
         i = i + 1;
       }
 
-      VariableBindings /*!*/
+      VariableBindings
         bindings = new VariableBindings();
 
       // type parameters that have to be given explicitly are replaced
       // with universally quantified type variables
-      List<VCExprVar /*!*/> /*!*/
-        boundVars = new List<VCExprVar /*!*/>(explicitTypeParams.Count + typedInputVars.Count);
-      foreach (TypeVariable /*!*/ var in explicitTypeParams)
+      List<VCExprVar>
+        boundVars = new List<VCExprVar>(explicitTypeParams.Count + typedInputVars.Count);
+      foreach (TypeVariable var in explicitTypeParams)
       {
         Contract.Assert(var != null);
-        VCExprVar /*!*/
+        VCExprVar
           newVar = Gen.Variable(var.Name, T);
         boundVars.Add(newVar);
         bindings.TypeVariableBindings.Add(var, newVar);
@@ -559,28 +555,28 @@ namespace Microsoft.Boogie.TypeErasure
 
       // bound term variables are replaced with bound term variables typed in
       // a simpler way
-      foreach (VCExprVar /*!*/ var in typedInputVars)
+      foreach (VCExprVar var in typedInputVars)
       {
         Contract.Assert(var != null);
-        Type /*!*/
+        Type
           newType = TypeAfterErasure(var.Type);
         Contract.Assert(newType != null);
-        VCExprVar /*!*/
+        VCExprVar
           newVar = Gen.Variable(var.Name, newType);
         Contract.Assert(newVar != null);
         boundVars.Add(newVar);
         bindings.VCExprVarBindings.Add(var, newVar);
       }
 
-      List<VCExprLetBinding /*!*/> typeVarBindings =
+      List<VCExprLetBinding> typeVarBindings =
         GenTypeParamBindings(implicitTypeParams, typedInputVars, bindings, true);
       Contract.Assert(Cce.NonNullElements(typeVarBindings));
       List<VCExpr> boundVarExprs = HelperFuns.ToVCExprList(boundVars);
 
-      VCExpr /*!*/
+      VCExpr
         funApp = Gen.Function(fun, boundVarExprs);
       Contract.Assert(funApp != null);
-      VCExpr /*!*/
+      VCExpr
         conclusion = Gen.Eq(TypeOf(funApp),
           Type2Term(originalResultType, bindings.TypeVariableBindings));
       Contract.Assert(conclusion != null);
@@ -596,7 +592,7 @@ namespace Microsoft.Boogie.TypeErasure
 
       if (boundVars.Count > 0)
       {
-        List<VCTrigger /*!*/> triggers = HelperFuns.ToList(Gen.Trigger(true, HelperFuns.ToList(funApp)));
+        List<VCTrigger> triggers = HelperFuns.ToList(Gen.Trigger(true, HelperFuns.ToList(funApp)));
         Contract.Assert(Cce.NonNullElements(triggers));
         return Gen.Forall(boundVars, triggers, "funType:" + fun.Name, 1, conclusionWithPremisses);
       }
@@ -610,15 +606,13 @@ namespace Microsoft.Boogie.TypeErasure
 
     protected override void AddVarTypeAxiom(VCExprVar var, Type originalType)
     {
-      //Contract.Requires(originalType != null);
-      //Contract.Requires(var != null);
       AddTypeAxiom(GenVarTypeAxiom(var, originalType,
         // we don't have any bindings available
-        new Dictionary<TypeVariable /*!*/, VCExpr /*!*/>()));
+        new Dictionary<TypeVariable, VCExpr>()));
     }
 
     public VCExpr GenVarTypeAxiom(VCExprVar var, Type originalType,
-      IDictionary<TypeVariable /*!*/, VCExpr /*!*/> /*!*/ varMapping)
+      IDictionary<TypeVariable, VCExpr> varMapping)
     {
       Contract.Requires(var != null);
       Contract.Requires(originalType != null);
@@ -627,7 +621,7 @@ namespace Microsoft.Boogie.TypeErasure
 
       if (!var.Type.Equals(originalType))
       {
-        VCExpr /*!*/
+        VCExpr
           typeRepr = Type2Term(originalType, varMapping);
         return Gen.Eq(TypeOf(var), typeRepr);
       }
@@ -640,7 +634,7 @@ namespace Microsoft.Boogie.TypeErasure
 
   internal class MapTypeAbstractionBuilderPremisses : MapTypeAbstractionBuilder
   {
-    private readonly TypeAxiomBuilderPremisses /*!*/ AxBuilderPremisses;
+    private readonly TypeAxiomBuilderPremisses AxBuilderPremisses;
 
     [ContractInvariantMethod]
     void ObjectInvariant()
@@ -678,7 +672,7 @@ namespace Microsoft.Boogie.TypeErasure
     // map). These parameters are given as a list of indexes sorted in
     // ascending order; the index i refers to the i'th bound variable
     // in a type    <a0, a1, ..., an>[...]...
-    public List<int> /*!*/ ExplicitSelectTypeParams(MapType type)
+    public List<int> ExplicitSelectTypeParams(MapType type)
     {
       Contract.Requires(type != null);
       Contract.Ensures(Contract.Result<List<int>>() != null);
@@ -690,7 +684,7 @@ namespace Microsoft.Boogie.TypeErasure
           out var implicitParams,
           out var explicitParams);
         res = new List<int>(explicitParams.Count);
-        foreach (TypeVariable /*!*/ var in explicitParams)
+        foreach (TypeVariable var in explicitParams)
         {
           Contract.Assert(var != null);
           res.Add(type.TypeParameters.IndexOf(var));
@@ -702,9 +696,9 @@ namespace Microsoft.Boogie.TypeErasure
       return Cce.NonNull(res);
     }
 
-    private IDictionary<MapType /*!*/, List<int> /*!*/> /*!*/
+    private IDictionary<MapType, List<int>>
       explicitSelectTypeParamsCache =
-        new Dictionary<MapType /*!*/, List<int> /*!*/>();
+        new Dictionary<MapType, List<int>>();
 
     [ContractInvariantMethod]
     void ObjectInvarant()
@@ -716,10 +710,8 @@ namespace Microsoft.Boogie.TypeErasure
     ////////////////////////////////////////////////////////////////////////////
 
     protected override void GenSelectStoreFunctions(MapType abstractedType, TypeCtorDecl synonym,
-      out Function /*!*/ select, out Function /*!*/ store)
+      out Function select, out Function store)
     {
-      //Contract.Requires(synonym != null);
-      //Contract.Requires(abstractedType != null);
       Contract.Ensures(Contract.ValueAtReturn(out select) != null);
       Contract.Ensures(Contract.ValueAtReturn(out store) != null);
       GenTypeAxiomParams(abstractedType, synonym, out var mapTypeSynonym,
@@ -745,24 +737,24 @@ namespace Microsoft.Boogie.TypeErasure
       AxBuilder.AddTypeAxiom(GenMapAxiom1(select, store, abstractedType.Result, explicitSelectParams));
     }
 
-    protected void GenTypeAxiomParams(MapType /*!*/ abstractedType, TypeCtorDecl /*!*/ synonymDecl,
-      out Type /*!*/ mapTypeSynonym,
-      out List<TypeVariable /*!*/> /*!*/ typeParams,
-      out List<Type /*!*/> /*!*/ originalIndexTypes)
+    protected void GenTypeAxiomParams(MapType abstractedType, TypeCtorDecl synonymDecl,
+      out Type mapTypeSynonym,
+      out List<TypeVariable> typeParams,
+      out List<Type> originalIndexTypes)
     {
       Contract.Requires(abstractedType != null);
       Contract.Requires(synonymDecl != null);
       Contract.Ensures(Contract.ValueAtReturn(out mapTypeSynonym) != null);
       Contract.Ensures(Cce.NonNullElements(Contract.ValueAtReturn(out typeParams)));
       Contract.Ensures(Cce.NonNullElements(Contract.ValueAtReturn(out originalIndexTypes)));
-      typeParams = new List<TypeVariable /*!*/>();
+      typeParams = new List<TypeVariable>();
       typeParams.AddRange(abstractedType.TypeParameters);
       typeParams.AddRange(abstractedType.FreeVariables);
 
-      originalIndexTypes = new List<Type /*!*/>(abstractedType.Arguments.Count + 1);
-      List<Type> /*!*/
+      originalIndexTypes = new List<Type>(abstractedType.Arguments.Count + 1);
+      List<Type>
         mapTypeParams = new List<Type>();
-      foreach (TypeVariable /*!*/ var in abstractedType.FreeVariables)
+      foreach (TypeVariable var in abstractedType.FreeVariables)
       {
         Contract.Assert(var != null);
         mapTypeParams.Add(var);
@@ -775,11 +767,11 @@ namespace Microsoft.Boogie.TypeErasure
     }
 
     // method to actually create the select or store function
-    private Function /*!*/ CreateAccessFun(List<TypeVariable /*!*/> /*!*/ originalTypeParams,
-      List<Type /*!*/> /*!*/ originalInTypes,
-      Type /*!*/ originalResult,
-      string /*!*/ name,
-      out List<TypeVariable /*!*/> /*!*/ implicitTypeParams, out List<TypeVariable /*!*/> /*!*/ explicitTypeParams)
+    private Function CreateAccessFun(List<TypeVariable> originalTypeParams,
+      List<Type> originalInTypes,
+      Type originalResult,
+      string name,
+      out List<TypeVariable> implicitTypeParams, out List<TypeVariable> explicitTypeParams)
     {
       Contract.Requires(Cce.NonNullElements(originalTypeParams));
       Contract.Requires(Cce.NonNullElements(originalInTypes));
@@ -797,7 +789,7 @@ namespace Microsoft.Boogie.TypeErasure
         out implicitTypeParams,
         out explicitTypeParams);
 
-      Type[] /*!*/
+      Type[]
         ioTypes = new Type[explicitTypeParams.Count + originalInTypes.Count + 1];
       int i = 0;
       for (; i < explicitTypeParams.Count; ++i)
@@ -805,7 +797,7 @@ namespace Microsoft.Boogie.TypeErasure
         ioTypes[i] = AxBuilder.T;
       }
 
-      foreach (Type /*!*/ type in originalInTypes)
+      foreach (Type type in originalInTypes)
       {
         Contract.Assert(type != null);
         ioTypes[i] = AxBuilder.U;
@@ -814,7 +806,7 @@ namespace Microsoft.Boogie.TypeErasure
 
       ioTypes[i] = AxBuilder.U;
 
-      Function /*!*/
+      Function
         res = HelperFuns.BoogieFunction(name, ioTypes);
       Contract.Assert(res != null);
 
@@ -832,14 +824,14 @@ namespace Microsoft.Boogie.TypeErasure
     ///////////////////////////////////////////////////////////////////////////
     // The normal axioms of the theory of arrays (without extensionality)
 
-    private VCExpr /*!*/ Select(Function /*!*/ select,
+    private VCExpr Select(Function select,
       // in general, the select function has to
       // receive explicit type parameters (which
       // are here already represented as VCExpr
       // of type T)
-      List<VCExpr /*!*/> /*!*/ typeParams,
-      VCExpr /*!*/ map,
-      List<VCExprVar /*!*/> /*!*/ indexes)
+      List<VCExpr> typeParams,
+      VCExpr map,
+      List<VCExprVar> indexes)
     {
       Contract.Requires(select != null);
       Contract.Requires(Cce.NonNullElements(typeParams));
@@ -847,23 +839,23 @@ namespace Microsoft.Boogie.TypeErasure
       Contract.Requires(Cce.NonNullElements(indexes));
       Contract.Ensures(Contract.Result<VCExpr>() != null);
 
-      List<VCExpr /*!*/> /*!*/
-        selectArgs = new List<VCExpr /*!*/>(typeParams.Count + indexes.Count + 1);
+      List<VCExpr>
+        selectArgs = new List<VCExpr>(typeParams.Count + indexes.Count + 1);
       selectArgs.AddRange(typeParams);
       selectArgs.Add(map);
       selectArgs.AddRange(HelperFuns.ToVCExprList(indexes));
       return Gen.Function(select, selectArgs);
     }
 
-    private VCExpr Store(Function store, VCExpr map, List<VCExprVar /*!*/> /*!*/ indexes, VCExpr val)
+    private VCExpr Store(Function store, VCExpr map, List<VCExprVar> indexes, VCExpr val)
     {
       Contract.Requires(val != null);
       Contract.Requires(map != null);
       Contract.Requires(store != null);
       Contract.Requires(Cce.NonNullElements(indexes));
       Contract.Ensures(Contract.Result<VCExpr>() != null);
-      List<VCExpr /*!*/> /*!*/
-        storeArgs = new List<VCExpr /*!*/>(indexes.Count + 2);
+      List<VCExpr>
+        storeArgs = new List<VCExpr>(indexes.Count + 2);
       storeArgs.Add(map);
       storeArgs.AddRange(HelperFuns.ToVCExprList(indexes));
       storeArgs.Add(val);
@@ -878,8 +870,8 @@ namespace Microsoft.Boogie.TypeErasure
     /// where the quantifier body is also enclosed in a let that defines portions of T, if needed.
     /// </summary>
     private VCExpr GenMapAxiom0(Function select, Function store, Type mapResult,
-      List<TypeVariable /*!*/> /*!*/ implicitTypeParamsSelect, List<TypeVariable /*!*/> /*!*/ explicitTypeParamsSelect,
-      List<Type /*!*/> /*!*/ originalInTypes)
+      List<TypeVariable> implicitTypeParamsSelect, List<TypeVariable> explicitTypeParamsSelect,
+      List<Type> originalInTypes)
     {
       Contract.Requires(mapResult != null);
       Contract.Requires(store != null);
@@ -889,8 +881,8 @@ namespace Microsoft.Boogie.TypeErasure
       Contract.Requires(Cce.NonNullElements(explicitTypeParamsSelect));
       Contract.Ensures(Contract.Result<VCExpr>() != null);
       int arity = store.InParams.Count - 2;
-      List<VCExprVar /*!*/> inParams = new List<VCExprVar /*!*/>();
-      List<VCExprVar /*!*/> quantifiedVars = new List<VCExprVar /*!*/>(store.InParams.Count);
+      List<VCExprVar> inParams = new List<VCExprVar>();
+      List<VCExprVar> quantifiedVars = new List<VCExprVar>(store.InParams.Count);
       VariableBindings bindings = new VariableBindings();
 
       // bound variable:  m
@@ -901,8 +893,8 @@ namespace Microsoft.Boogie.TypeErasure
       bindings.VCExprVarBindings.Add(typedM, m);
 
       // bound variables:  indexes
-      List<Type /*!*/> origIndexTypes = new List<Type /*!*/>(arity);
-      List<Type /*!*/> indexTypes = new List<Type /*!*/>(arity);
+      List<Type> origIndexTypes = new List<Type>(arity);
+      List<Type> indexTypes = new List<Type>(arity);
       for (int i = 1; i < store.InParams.Count - 1; i++)
       {
         origIndexTypes.Add(originalInTypes[i]);
@@ -910,9 +902,9 @@ namespace Microsoft.Boogie.TypeErasure
       }
 
       Contract.Assert(arity == indexTypes.Count);
-      List<VCExprVar /*!*/> typedArgs = HelperFuns.VarVector("arg", origIndexTypes, Gen);
+      List<VCExprVar> typedArgs = HelperFuns.VarVector("arg", origIndexTypes, Gen);
       Contract.Assert(Cce.NonNullElements(typedArgs));
-      List<VCExprVar /*!*/> indexes = HelperFuns.VarVector("x", indexTypes, Gen);
+      List<VCExprVar> indexes = HelperFuns.VarVector("x", indexTypes, Gen);
       Contract.Assert(Cce.NonNullElements(indexes));
       Contract.Assert(typedArgs.Count == indexes.Count);
       inParams.AddRange(typedArgs);
@@ -935,7 +927,7 @@ namespace Microsoft.Boogie.TypeErasure
         bindings.TypeVariableBindings.Add(tp, tVar);
       }
 
-      List<VCExpr /*!*/> typeParams = new List<VCExpr /*!*/>(explicitTypeParamsSelect.Count);
+      List<VCExpr> typeParams = new List<VCExpr>(explicitTypeParamsSelect.Count);
       foreach (TypeVariable tp in explicitTypeParamsSelect)
       {
         VCExprVar tVar = Gen.Variable(tp.Name, AxBuilderPremisses.T);
@@ -944,10 +936,10 @@ namespace Microsoft.Boogie.TypeErasure
         typeParams.Add(tVar);
       }
 
-      VCExpr /*!*/
+      VCExpr
         storeExpr = Store(store, m, indexes, val);
       Contract.Assert(storeExpr != null);
-      VCExpr /*!*/
+      VCExpr
         selectExpr = Select(select, typeParams, storeExpr, indexes);
       Contract.Assert(selectExpr != null);
 
@@ -955,21 +947,21 @@ namespace Microsoft.Boogie.TypeErasure
       // The implicit ones can be phrased in terms of the types of the ordinary in-parameters, and
       // we want to make sure that they don't get phrased in terms of the out-parameter, so we pass
       // in inParams here.
-      List<VCExprLetBinding /*!*/> letBindings_Implicit =
+      List<VCExprLetBinding> letBindings_Implicit =
         AxBuilderPremisses.GenTypeParamBindings(implicitTypeParamsSelect, inParams, bindings, false);
       Contract.Assert(Cce.NonNullElements(letBindings_Implicit));
       // The explicit ones, by definition, can only be phrased in terms of the result, so we pass
       // in List(typedVal) here.
-      List<VCExprLetBinding /*!*/> letBindings_Explicit =
+      List<VCExprLetBinding> letBindings_Explicit =
         AxBuilderPremisses.GenTypeParamBindings(explicitTypeParamsSelect, HelperFuns.ToList(typedVal), bindings, false);
       Contract.Assert(Cce.NonNullElements(letBindings_Explicit));
 
       // generate:  select(store(m, indices, val)) == val
-      VCExpr /*!*/
+      VCExpr
         eq = Gen.Eq(selectExpr, val);
       Contract.Assert(eq != null);
       // generate:  type(val) == T, where T is the type of val
-      VCExpr /*!*/
+      VCExpr
         ante = Gen.Eq(
           AxBuilderPremisses.TypeOf(val),
           AxBuilderPremisses.Type2Term(mapResult, bindings.TypeVariableBindings));
@@ -984,11 +976,11 @@ namespace Microsoft.Boogie.TypeErasure
         body = Gen.Let(letBindings_Implicit, Gen.Let(letBindings_Explicit, Gen.ImpliesSimp(ante, eq)));
       }
 
-      return Gen.Forall(quantifiedVars, new List<VCTrigger /*!*/>(), "mapAx0:" + select.Name, 0, body);
+      return Gen.Forall(quantifiedVars, new List<VCTrigger>(), "mapAx0:" + select.Name, 0, body);
     }
 
     private VCExpr GenMapAxiom1(Function select, Function store, Type mapResult,
-      List<TypeVariable /*!*/> /*!*/ explicitSelectParams)
+      List<TypeVariable> explicitSelectParams)
     {
       Contract.Requires(mapResult != null);
       Contract.Requires(store != null);
@@ -997,7 +989,7 @@ namespace Microsoft.Boogie.TypeErasure
       Contract.Ensures(Contract.Result<VCExpr>() != null);
       int arity = store.InParams.Count - 2;
 
-      List<Type /*!*/> indexTypes = new List<Type /*!*/>();
+      List<Type> indexTypes = new List<Type>();
       for (int i = 1; i < store.InParams.Count - 1; i++)
       {
         indexTypes.Add(Cce.NonNull(store.InParams[i]).TypedIdent.Type);
@@ -1005,28 +997,28 @@ namespace Microsoft.Boogie.TypeErasure
 
       Contract.Assert(indexTypes.Count == arity);
 
-      List<VCExprVar /*!*/> /*!*/
+      List<VCExprVar>
         indexes0 = HelperFuns.VarVector("x", indexTypes, Gen);
       Contract.Assert(Cce.NonNullElements(indexes0));
-      List<VCExprVar /*!*/> /*!*/
+      List<VCExprVar>
         indexes1 = HelperFuns.VarVector("y", indexTypes, Gen);
       Contract.Assert(Cce.NonNullElements(indexes1));
-      VCExprVar /*!*/
+      VCExprVar
         m = Gen.Variable("m", AxBuilder.U);
       Contract.Assert(m != null);
-      VCExprVar /*!*/
+      VCExprVar
         val = Gen.Variable("val", Cce.NonNull(select.OutParams[0]).TypedIdent.Type);
       Contract.Assert(val != null);
 
       // extract the explicit type parameters from the actual result type ...
-      VCExprVar /*!*/
+      VCExprVar
         typedVal = Gen.Variable("val", mapResult);
       Contract.Assert(typedVal != null);
-      VariableBindings /*!*/
+      VariableBindings
         bindings = new VariableBindings();
       bindings.VCExprVarBindings.Add(typedVal, val);
 
-      List<VCExprLetBinding /*!*/> /*!*/
+      List<VCExprLetBinding>
         letBindings =
           AxBuilderPremisses.GenTypeParamBindings(explicitSelectParams,
             HelperFuns.ToList(typedVal),
@@ -1035,47 +1027,47 @@ namespace Microsoft.Boogie.TypeErasure
 
       // ... and quantify the introduced term variables for type
       // parameters universally
-      List<VCExprVar /*!*/> /*!*/
-        typeParams = new List<VCExprVar /*!*/>(explicitSelectParams.Count);
-      List<VCExpr /*!*/> /*!*/
-        typeParamsExpr = new List<VCExpr /*!*/>(explicitSelectParams.Count);
-      foreach (TypeVariable /*!*/ var in explicitSelectParams)
+      List<VCExprVar>
+        typeParams = new List<VCExprVar>(explicitSelectParams.Count);
+      List<VCExpr>
+        typeParamsExpr = new List<VCExpr>(explicitSelectParams.Count);
+      foreach (TypeVariable var in explicitSelectParams)
       {
         Contract.Assert(var != null);
-        VCExprVar /*!*/
+        VCExprVar
           newVar = (VCExprVar) bindings.TypeVariableBindings[var];
         Contract.Assert(newVar != null);
         typeParams.Add(newVar);
         typeParamsExpr.Add(newVar);
       }
 
-      VCExpr /*!*/
+      VCExpr
         storeExpr = Store(store, m, indexes0, val);
       Contract.Assert(storeExpr != null);
-      VCExpr /*!*/
+      VCExpr
         selectWithoutStoreExpr = Select(select, typeParamsExpr, m, indexes1);
       Contract.Assert(selectWithoutStoreExpr != null);
-      VCExpr /*!*/
+      VCExpr
         selectExpr = Select(select, typeParamsExpr, storeExpr, indexes1);
       Contract.Assert(selectExpr != null);
 
-      VCExpr /*!*/
+      VCExpr
         selectEq = Gen.Eq(selectExpr, selectWithoutStoreExpr);
       Contract.Assert(selectEq != null);
 
-      List<VCExprVar /*!*/> /*!*/
-        quantifiedVars = new List<VCExprVar /*!*/>(indexes0.Count + indexes1.Count + 2);
+      List<VCExprVar>
+        quantifiedVars = new List<VCExprVar>(indexes0.Count + indexes1.Count + 2);
       quantifiedVars.Add(val);
       quantifiedVars.Add(m);
       quantifiedVars.AddRange(indexes0);
       quantifiedVars.AddRange(indexes1);
       quantifiedVars.AddRange(typeParams);
 
-      List<VCTrigger /*!*/> /*!*/
-        triggers = new List<VCTrigger /*!*/>();
+      List<VCTrigger>
+        triggers = new List<VCTrigger>();
       Contract.Assert(Cce.NonNullElements(triggers));
 
-      VCExpr /*!*/
+      VCExpr
         axiom = VCExpressionGenerator.True;
       Contract.Assert(axiom != null);
 
@@ -1083,11 +1075,11 @@ namespace Microsoft.Boogie.TypeErasure
       // different from the assigned location
       for (int i = 0; i < arity; ++i)
       {
-        VCExpr /*!*/
+        VCExpr
           indexesEq = Gen.Eq(indexes0[i], indexes1[i]);
-        VCExpr /*!*/
+        VCExpr
           matrix = Gen.Or(indexesEq, selectEq);
-        VCExpr /*!*/
+        VCExpr
           conjunct = Gen.Forall(quantifiedVars, triggers, "mapAx1:" + select.Name + ":" + i, 0, matrix);
         Contract.Assert(indexesEq != null);
         Contract.Assert(matrix != null);
@@ -1097,17 +1089,17 @@ namespace Microsoft.Boogie.TypeErasure
 
       // second non-interference criterion: the queried type is
       // different from the assigned type
-      VCExpr /*!*/
+      VCExpr
         typesEq = VCExpressionGenerator.True;
-      foreach (VCExprLetBinding /*!*/ b in letBindings)
+      foreach (VCExprLetBinding b in letBindings)
       {
         Contract.Assert(b != null);
         typesEq = Gen.AndSimp(typesEq, Gen.Eq(b.V, b.E));
       }
 
-      VCExpr /*!*/
+      VCExpr
         matrix2 = Gen.Or(typesEq, selectEq);
-      VCExpr /*!*/
+      VCExpr
         conjunct2 = Gen.Forall(quantifiedVars, triggers, "mapAx2:" + select.Name, 0, matrix2);
       axiom = Gen.AndSimp(axiom, conjunct2);
 
