@@ -58,7 +58,7 @@ namespace Microsoft.Boogie.VCExprAST
     {
       Contract.Requires(l != null);
       Contract.Ensures(Contract.Result<List<T>>() != null);
-      List<T> /*!*/
+      List<T>
         res = new List<T>();
       foreach (T x in l)
       {
@@ -73,7 +73,7 @@ namespace Microsoft.Boogie.VCExprAST
       Contract.Requires(exprs != null);
       Contract.Requires((Contract.ForAll(0, exprs.Length, i => exprs[i] != null)));
       Contract.Ensures(Contract.Result<List<Type>>() != null);
-      List<Type> /*!*/
+      List<Type>
         res = new List<Type>();
       for (int i = startIndex; i < exprs.Length; ++i)
       {
@@ -83,10 +83,10 @@ namespace Microsoft.Boogie.VCExprAST
       return res;
     }
 
-    public static List<T /*!*/> /*!*/ ToNonNullList<T>(params T[] args) where T : class
+    public static List<T> ToNonNullList<T>(params T[] args) where T : class
     {
       Contract.Requires(args != null);
-      List<T /*!*/> /*!*/
+      List<T>
         res = new List<T>(args.Length);
       foreach (T t in args)
       {
@@ -173,7 +173,6 @@ namespace Microsoft.Boogie.VCExprAST
 
     public override Result Accept<Result, Arg>(IVCExprVisitor<Result, Arg> visitor, Arg arg)
     {
-      //Contract.Requires(visitor != null);
       return visitor.Visit(this, arg);
     }
   }
@@ -416,10 +415,10 @@ namespace Microsoft.Boogie.VCExprAST
     }
 
     // the sub-expressions of the expression
-    public abstract VCExpr /*!*/ this[int index] { get; }
+    public abstract VCExpr this[int index] { get; }
 
     // the type arguments
-    public abstract List<Type /*!*/> /*!*/ TypeArguments { get; }
+    public abstract List<Type> TypeArguments { get; }
 
     [Pure]
     [Reads(ReadsAttribute.Reads.Nothing)]
@@ -494,7 +493,6 @@ namespace Microsoft.Boogie.VCExprAST
 
     public override Result Accept<Result, Arg>(IVCExprVisitor<Result, Arg> visitor, Arg arg)
     {
-      //Contract.Requires(visitor != null);
       return visitor.Visit(this, arg);
     }
 
@@ -504,11 +502,11 @@ namespace Microsoft.Boogie.VCExprAST
       return Op.Accept(this, visitor, arg);
     }
 
-    internal static readonly List<Type /*!*/> /*!*/
-      EMPTY_TYPE_LIST = new List<Type /*!*/>();
+    internal static readonly List<Type>
+      EMPTY_TYPE_LIST = new List<Type>();
 
-    internal static readonly List<VCExpr /*!*/> /*!*/
-      EMPTY_VCEXPR_LIST = new List<VCExpr /*!*/>();
+    internal static readonly List<VCExpr>
+      EMPTY_VCEXPR_LIST = new List<VCExpr>();
 
     public IEnumerable<VCExpr> UniformArguments
     {
@@ -560,7 +558,7 @@ namespace Microsoft.Boogie.VCExprAST
     }
 
     // the type arguments
-    public override List<Type /*!*/> /*!*/ TypeArguments
+    public override List<Type> TypeArguments
     {
       get
       {
@@ -587,13 +585,13 @@ namespace Microsoft.Boogie.VCExprAST
       Contract.Invariant(ExprType != null);
     }
 
-    private readonly VCExpr /*!*/
+    private readonly VCExpr
       Argument;
 
-    private readonly Type /*!*/
+    private readonly Type
       ExprType;
 
-    public override Type /*!*/ Type
+    public override Type Type
     {
       get
       {
@@ -602,7 +600,7 @@ namespace Microsoft.Boogie.VCExprAST
       }
     }
 
-    public override VCExpr /*!*/ this[int index]
+    public override VCExpr this[int index]
     {
       get
       {
@@ -614,7 +612,7 @@ namespace Microsoft.Boogie.VCExprAST
     }
 
     // the type arguments
-    public override List<Type /*!*/> /*!*/ TypeArguments
+    public override List<Type> TypeArguments
     {
       get
       {
@@ -695,7 +693,7 @@ namespace Microsoft.Boogie.VCExprAST
     }
 
     // the type arguments
-    public override List<Type /*!*/> /*!*/ TypeArguments
+    public override List<Type> TypeArguments
     {
       get
       {
@@ -735,9 +733,9 @@ namespace Microsoft.Boogie.VCExprAST
 
   internal class VCExprMultiAry : VCExprNAry
   {
-    private readonly List<VCExpr /*!*/> /*!*/ arguments;
+    private readonly List<VCExpr> arguments;
 
-    private readonly List<Type /*!*/> /*!*/
+    private readonly List<Type>
       TypeArgumentsAttr;
 
     [ContractInvariantMethod]
@@ -749,10 +747,10 @@ namespace Microsoft.Boogie.VCExprAST
     }
 
 
-    private readonly Type /*!*/
+    private readonly Type
       ExprType;
 
-    public override Type /*!*/ Type
+    public override Type Type
     {
       get
       {
@@ -761,7 +759,7 @@ namespace Microsoft.Boogie.VCExprAST
       }
     }
 
-    public override VCExpr /*!*/ this[int index]
+    public override VCExpr this[int index]
     {
       get
       {
@@ -773,7 +771,7 @@ namespace Microsoft.Boogie.VCExprAST
     }
 
     // the type arguments
-    public override List<Type /*!*/> /*!*/ TypeArguments
+    public override List<Type> TypeArguments
     {
       get
       {
@@ -792,7 +790,7 @@ namespace Microsoft.Boogie.VCExprAST
       this.ExprType = op.InferType(arguments, TypeArgumentsAttr);
     }
 
-    internal VCExprMultiAry(VCExprOp op, List<VCExpr> arguments, List<Type /*!*/> /*!*/ typeArguments)
+    internal VCExprMultiAry(VCExprOp op, List<VCExpr> arguments, List<Type> typeArguments)
       : base(op)
     {
       Contract.Requires(op != null);
@@ -818,7 +816,7 @@ namespace Microsoft.Boogie.VCExprAST
     // the number of type parameters
     public abstract int TypeParamArity { get; }
 
-    public abstract Type /*!*/ InferType(List<VCExpr /*!*/> /*!*/ args, List<Type /*!*/> /*!*/ typeArgs);
+    public abstract Type InferType(List<VCExpr> args, List<Type> typeArgs);
 
     public virtual Result Accept<Result, Arg>(VCExprNAry expr, IVCExprOpVisitor<Result, Arg> visitor, Arg arg)
     {
@@ -919,10 +917,8 @@ namespace Microsoft.Boogie.VCExprAST
       get { return 0; }
     }
 
-    public override Type InferType(List<VCExpr> args, List<Type /*!*/> /*!*/ typeArgs)
+    public override Type InferType(List<VCExpr> args, List<Type> typeArgs)
     {
-      //Contract.Requires(cce.NonNullElements(typeArgs));
-      //Contract.Requires(cce.NonNullElements(args));
       Contract.Ensures(Contract.Result<Type>() != null);
       return OpType;
     }
@@ -970,8 +966,6 @@ namespace Microsoft.Boogie.VCExprAST
       IVCExprOpVisitor<Result, Arg> visitor,
       Arg arg)
     {
-      //Contract.Requires(visitor != null);
-      //Contract.Requires(expr != null);
       return visitor.VisitDistinctOp(expr, arg);
     }
   }
@@ -1095,16 +1089,14 @@ namespace Microsoft.Boogie.VCExprAST
       get { return MapTypeParamArity; }
     }
 
-    public override Type InferType(List<VCExpr> args, List<Type /*!*/> /*!*/ typeArgs)
+    public override Type InferType(List<VCExpr> args, List<Type> typeArgs)
     {
-      //Contract.Requires(cce.NonNullElements(typeArgs));
-      //Contract.Requires(cce.NonNullElements(args));
       Contract.Ensures(Contract.Result<Type>() != null);
-      MapType /*!*/
+      MapType
         mapType = args[0].Type.AsMap;
       Contract.Assert(TypeParamArity == mapType.TypeParameters.Count);
-      IDictionary<TypeVariable /*!*/, Type /*!*/> /*!*/
-        subst = new Dictionary<TypeVariable /*!*/, Type /*!*/>();
+      IDictionary<TypeVariable, Type>
+        subst = new Dictionary<TypeVariable, Type>();
       for (int i = 0; i < TypeParamArity; ++i)
       {
         subst.Add(mapType.TypeParameters[i], typeArgs[i]);
@@ -1149,8 +1141,6 @@ namespace Microsoft.Boogie.VCExprAST
       IVCExprOpVisitor<Result, Arg> visitor,
       Arg arg)
     {
-      //Contract.Requires(visitor != null);
-      //Contract.Requires(expr != null);
       return visitor.VisitSelectOp(expr, arg);
     }
   }
@@ -1172,10 +1162,8 @@ namespace Microsoft.Boogie.VCExprAST
       get { return MapTypeParamArity; }
     }
 
-    public override Type InferType(List<VCExpr> args, List<Type /*!*/> /*!*/ typeArgs)
+    public override Type InferType(List<VCExpr> args, List<Type> typeArgs)
     {
-      //Contract.Requires(cce.NonNullElements(typeArgs));
-      //Contract.Requires(cce.NonNullElements(args));
       Contract.Ensures(Contract.Result<Type>() != null);
       return args[0].Type;
     }
@@ -1215,8 +1203,6 @@ namespace Microsoft.Boogie.VCExprAST
       IVCExprOpVisitor<Result, Arg> visitor,
       Arg arg)
     {
-      //Contract.Requires(visitor != null);
-      //Contract.Requires(expr != null);
       return visitor.VisitStoreOp(expr, arg);
     }
   }
@@ -1233,10 +1219,8 @@ namespace Microsoft.Boogie.VCExprAST
       get { return 0; }
     }
 
-    public override Type InferType(List<VCExpr> args, List<Type /*!*/> /*!*/ typeArgs)
+    public override Type InferType(List<VCExpr> args, List<Type> typeArgs)
     {
-      //Contract.Requires(cce.NonNullElements(typeArgs));
-      //Contract.Requires(cce.NonNullElements(args));
       Contract.Ensures(Contract.Result<Type>() != null);
       return args[1].Type;
     }
@@ -1273,8 +1257,6 @@ namespace Microsoft.Boogie.VCExprAST
       IVCExprOpVisitor<Result, Arg> visitor,
       Arg arg)
     {
-      //Contract.Requires(visitor != null);
-      //Contract.Requires(expr != null);
       return visitor.VisitIfThenElseOp(expr, arg);
     }
   }
@@ -1291,12 +1273,12 @@ namespace Microsoft.Boogie.VCExprAST
 
   public class VCExprCustomOp : VCExprOp
   {
-    public readonly string /*!*/
+    public readonly string
       Name;
 
     int arity;
 
-    public readonly Type /*!*/
+    public readonly Type
       Type;
 
     [ContractInvariantMethod]
@@ -1306,7 +1288,7 @@ namespace Microsoft.Boogie.VCExprAST
       Contract.Invariant(Type != null);
     }
 
-    public VCExprCustomOp(string /*!*/ name, int arity, Type /*!*/ type)
+    public VCExprCustomOp(string name, int arity, Type type)
     {
       Contract.Requires(name != null);
       Contract.Requires(type != null);
@@ -1353,21 +1335,17 @@ namespace Microsoft.Boogie.VCExprAST
       get { return 0; }
     }
 
-    public override Type /*!*/ InferType(List<VCExpr /*!*/> /*!*/ args, List<Type /*!*/> /*!*/ typeArgs)
+    public override Type InferType(List<VCExpr> args, List<Type> typeArgs)
     {
-      //Contract.Requires((cce.NonNullElements(args)));
-      //Contract.Requires((cce.NonNullElements(typeArgs)));
       Contract.Ensures(Contract.Result<Type>() != null);
       return Type;
     }
 
     public override Result Accept<Result, Arg>(
-      VCExprNAry /*!*/ expr,
-      IVCExprOpVisitor<Result, Arg> /*!*/ visitor,
+      VCExprNAry expr,
+      IVCExprOpVisitor<Result, Arg> visitor,
       Arg arg)
     {
-      //Contract.Requires(expr != null);
-      //Contract.Requires(visitor != null);
       return visitor.VisitCustomOp(expr, arg);
     }
   }
@@ -1391,10 +1369,8 @@ namespace Microsoft.Boogie.VCExprAST
       get { return 2; }
     }
 
-    public override Type InferType(List<VCExpr> args, List<Type /*!*/> /*!*/ typeArgs)
+    public override Type InferType(List<VCExpr> args, List<Type> typeArgs)
     {
-      //Contract.Requires(cce.NonNullElements(typeArgs));
-      //Contract.Requires(cce.NonNullElements(args));
       Contract.Ensures(Contract.Result<Type>() != null);
       switch (op)
       {
@@ -1452,8 +1428,6 @@ namespace Microsoft.Boogie.VCExprAST
       IVCExprOpVisitor<Result, Arg> visitor,
       Arg arg)
     {
-      //Contract.Requires(visitor != null);
-      //Contract.Requires(expr != null);
       switch (op)
       {
         case ("+"):
@@ -1500,10 +1474,8 @@ namespace Microsoft.Boogie.VCExprAST
       get { return 0; }
     }
 
-    public override Type InferType(List<VCExpr> args, List<Type /*!*/> /*!*/ typeArgs)
+    public override Type InferType(List<VCExpr> args, List<Type> typeArgs)
     {
-      //Contract.Requires(cce.NonNullElements(typeArgs));
-      //Contract.Requires(cce.NonNullElements(args));
       Contract.Ensures(Contract.Result<Type>() != null);
       return Type.GetBvType(Bits);
     }
@@ -1541,8 +1513,6 @@ namespace Microsoft.Boogie.VCExprAST
       IVCExprOpVisitor<Result, Arg> visitor,
       Arg arg)
     {
-      //Contract.Requires(visitor != null);
-      //Contract.Requires(expr != null);
       return visitor.VisitBvOp(expr, arg);
     }
   }
@@ -1563,10 +1533,8 @@ namespace Microsoft.Boogie.VCExprAST
       get { return 0; }
     }
 
-    public override Type InferType(List<VCExpr> args, List<Type /*!*/> /*!*/ typeArgs)
+    public override Type InferType(List<VCExpr> args, List<Type> typeArgs)
     {
-      //Contract.Requires(cce.NonNullElements(typeArgs));
-      //Contract.Requires(cce.NonNullElements(args));
       Contract.Ensures(Contract.Result<Type>() != null);
       return Type.GetBvType(End - Start);
     }
@@ -1582,7 +1550,7 @@ namespace Microsoft.Boogie.VCExprAST
 
       if (that is VCExprBvExtractOp)
       {
-        VCExprBvExtractOp /*!*/
+        VCExprBvExtractOp
           thatExtract = (VCExprBvExtractOp) that;
         return this.Start == thatExtract.Start && this.End == thatExtract.End && this.Total == thatExtract.Total;
       }
@@ -1609,8 +1577,6 @@ namespace Microsoft.Boogie.VCExprAST
       IVCExprOpVisitor<Result, Arg> visitor,
       Arg arg)
     {
-      //Contract.Requires(visitor != null);
-      //Contract.Requires(expr != null);
       return visitor.VisitBvExtractOp(expr, arg);
     }
   }
@@ -1630,10 +1596,8 @@ namespace Microsoft.Boogie.VCExprAST
       get { return 0; }
     }
 
-    public override Type InferType(List<VCExpr> args, List<Type /*!*/> /*!*/ typeArgs)
+    public override Type InferType(List<VCExpr> args, List<Type> typeArgs)
     {
-      //Contract.Requires(cce.NonNullElements(typeArgs));
-      //Contract.Requires(cce.NonNullElements(args));
       Contract.Ensures(Contract.Result<Type>() != null);
       return Type.GetBvType(args[0].Type.BvBits + args[1].Type.BvBits);
     }
@@ -1674,8 +1638,6 @@ namespace Microsoft.Boogie.VCExprAST
       IVCExprOpVisitor<Result, Arg> visitor,
       Arg arg)
     {
-      //Contract.Requires(visitor != null);
-      //Contract.Requires(expr != null);
       return visitor.VisitBvConcatOp(expr, arg);
     }
   }
@@ -1704,10 +1666,8 @@ namespace Microsoft.Boogie.VCExprAST
       get { return Func.TypeParameters.Count; }
     }
 
-    public override Type InferType(List<VCExpr> args, List<Type /*!*/> /*!*/ typeArgs)
+    public override Type InferType(List<VCExpr> args, List<Type> typeArgs)
     {
-      //Contract.Requires(cce.NonNullElements(typeArgs));
-      //Contract.Requires(cce.NonNullElements(args));
       Contract.Ensures(Contract.Result<Type>() != null);
       Contract.Assert(TypeParamArity == Func.TypeParameters.Count);
       if (TypeParamArity == 0)
@@ -1715,8 +1675,8 @@ namespace Microsoft.Boogie.VCExprAST
         return Cce.NonNull(Func.OutParams[0]).TypedIdent.Type;
       }
 
-      IDictionary<TypeVariable /*!*/, Type /*!*/> /*!*/
-        subst = new Dictionary<TypeVariable /*!*/, Type /*!*/>(TypeParamArity);
+      IDictionary<TypeVariable, Type>
+        subst = new Dictionary<TypeVariable, Type>(TypeParamArity);
       for (int i = 0; i < TypeParamArity; ++i)
       {
         subst.Add(Func.TypeParameters[i], typeArgs[i]);
@@ -1761,8 +1721,6 @@ namespace Microsoft.Boogie.VCExprAST
       IVCExprOpVisitor<Result, Arg> visitor,
       Arg arg)
     {
-      //Contract.Requires(visitor != null);
-      //Contract.Requires(expr != null);
       return visitor.VisitBoogieFunctionOp(expr, arg);
     }
 
@@ -1796,15 +1754,15 @@ namespace Microsoft.Boogie.VCExprAST
       Contract.Invariant(VarType != null);
     }
 
-    public readonly string /*!*/
+    public readonly string
       Name;
 
-    private readonly Type /*!*/
+    private readonly Type
       VarType;
 
     public VCExprVarKind VarKind { get; }
 
-    public override Type /*!*/ Type
+    public override Type Type
     {
       get
       {
@@ -1837,7 +1795,6 @@ namespace Microsoft.Boogie.VCExprAST
 
     public override Result Accept<Result, Arg>(IVCExprVisitor<Result, Arg> visitor, Arg arg)
     {
-      //Contract.Requires(visitor != null);
       return visitor.Visit(this, arg);
     }
     
@@ -1858,13 +1815,13 @@ namespace Microsoft.Boogie.VCExprAST
 
   public abstract class VCExprBinder : VCExpr
   {
-    public readonly VCExpr /*!*/
+    public readonly VCExpr
       Body;
 
-    public readonly List<TypeVariable /*!*/> /*!*/
+    public readonly List<TypeVariable>
       TypeParameters;
 
-    public readonly List<VCExprVar /*!*/> /*!*/
+    public readonly List<VCExprVar>
       BoundVars;
 
     [ContractInvariantMethod]
@@ -1876,7 +1833,7 @@ namespace Microsoft.Boogie.VCExprAST
     }
 
 
-    public override Type /*!*/ Type
+    public override Type Type
     {
       get
       {
@@ -1886,7 +1843,7 @@ namespace Microsoft.Boogie.VCExprAST
       }
     }
 
-    internal VCExprBinder(List<TypeVariable /*!*/> /*!*/ typeParams, List<VCExprVar /*!*/> /*!*/ boundVars, VCExpr body)
+    internal VCExprBinder(List<TypeVariable> typeParams, List<VCExprVar> boundVars, VCExpr body)
     {
       Contract.Requires(body != null);
       Contract.Requires(Cce.NonNullElements(boundVars));
@@ -1902,7 +1859,7 @@ namespace Microsoft.Boogie.VCExprAST
   {
     public readonly bool Pos;
 
-    public readonly List<VCExpr /*!*/> /*!*/
+    public readonly List<VCExpr>
       Exprs;
 
     [ContractInvariantMethod]
@@ -1923,7 +1880,7 @@ namespace Microsoft.Boogie.VCExprAST
 
       if (that is VCTrigger)
       {
-        VCTrigger /*!*/
+        VCTrigger
           thatTrigger = (VCTrigger) that;
         return this.Pos == thatTrigger.Pos &&
                HelperFuns.SameElements(this.Exprs, thatTrigger.Exprs);
@@ -2010,7 +1967,7 @@ namespace Microsoft.Boogie.VCExprAST
       Contract.Invariant(Cce.NonNullElements(Triggers));
     }
 
-    public readonly List<VCTrigger /*!*/> /*!*/
+    public readonly List<VCTrigger>
       Triggers;
 
     public readonly VCQuantifierInfo Info;
@@ -2027,7 +1984,7 @@ namespace Microsoft.Boogie.VCExprAST
 
       if (that is VCExprQuantifier)
       {
-        VCExprQuantifier /*!*/
+        VCExprQuantifier
           thatQuan = (VCExprQuantifier) that;
         return this.Quan == thatQuan.Quan &&
                HelperFuns.SameElements(this.Triggers, thatQuan.Triggers) &&
@@ -2048,8 +2005,8 @@ namespace Microsoft.Boogie.VCExprAST
              HelperFuns.PolyHash(123, 11, Triggers);
     }
 
-    internal VCExprQuantifier(Quantifier kind, List<TypeVariable /*!*/> /*!*/ typeParams,
-      List<VCExprVar /*!*/> /*!*/ boundVars, List<VCTrigger /*!*/> /*!*/ triggers, VCQuantifierInfo info, VCExpr body)
+    internal VCExprQuantifier(Quantifier kind, List<TypeVariable> typeParams,
+      List<VCExprVar> boundVars, List<VCTrigger> triggers, VCQuantifierInfo info, VCExpr body)
       : base(typeParams, boundVars, body)
     {
       Contract.Requires(body != null);
@@ -2065,7 +2022,6 @@ namespace Microsoft.Boogie.VCExprAST
 
     public override Result Accept<Result, Arg>(IVCExprVisitor<Result, Arg> visitor, Arg arg)
     {
-      //Contract.Requires(visitor != null);
       return visitor.Visit(this, arg);
     }
   }
@@ -2097,7 +2053,7 @@ namespace Microsoft.Boogie.VCExprAST
 
       if (that is VCExprLetBinding)
       {
-        VCExprLetBinding /*!*/
+        VCExprLetBinding
           thatB = (VCExprLetBinding) that;
         return this.V.Equals(thatB.V) && this.E.Equals(thatB.E);
       }
@@ -2121,9 +2077,9 @@ namespace Microsoft.Boogie.VCExprAST
     }
   }
 
-  public class VCExprLet : VCExprBinder, IEnumerable<VCExprLetBinding /*!*/>
+  public class VCExprLet : VCExprBinder, IEnumerable<VCExprLetBinding>
   {
-    private readonly List<VCExprLetBinding /*!*/> /*!*/
+    private readonly List<VCExprLetBinding>
       Bindings;
 
     [ContractInvariantMethod]
@@ -2158,7 +2114,7 @@ namespace Microsoft.Boogie.VCExprAST
 
       if (that is VCExprLet)
       {
-        VCExprLet /*!*/
+        VCExprLet
           thatLet = (VCExprLet) that;
         return this.Body.Equals(thatLet.Body) &&
                HelperFuns.SameElements(this, (VCExprLet) that);
@@ -2176,7 +2132,7 @@ namespace Microsoft.Boogie.VCExprAST
     [Pure]
     [GlobalAccess(false)]
     [Escapes(true, false)]
-    public IEnumerator<VCExprLetBinding /*!*/> /*!*/ GetEnumerator()
+    public IEnumerator<VCExprLetBinding> GetEnumerator()
     {
       Contract.Ensures(Cce.NonNullElements(Contract.Result<IEnumerator<VCExprLetBinding>>()));
       return Bindings.GetEnumerator();
@@ -2191,12 +2147,12 @@ namespace Microsoft.Boogie.VCExprAST
       return Bindings.GetEnumerator();
     }
 
-    private static List<VCExprVar /*!*/> /*!*/ toSeq(List<VCExprLetBinding /*!*/> /*!*/ bindings)
+    private static List<VCExprVar> toSeq(List<VCExprLetBinding> bindings)
     {
       Contract.Requires(Cce.NonNullElements(bindings));
       Contract.Ensures(Cce.NonNullElements(Contract.Result<List<VCExprVar>>()));
       List<VCExprVar> res = new List<VCExprVar>();
-      foreach (VCExprLetBinding /*!*/ b in bindings)
+      foreach (VCExprLetBinding b in bindings)
       {
         res.Add(b.V);
       }
@@ -2204,8 +2160,8 @@ namespace Microsoft.Boogie.VCExprAST
       return res;
     }
 
-    internal VCExprLet(List<VCExprLetBinding /*!*/> /*!*/ bindings, VCExpr /*!*/ body)
-      : base(new List<TypeVariable /*!*/>(), toSeq(bindings), body)
+    internal VCExprLet(List<VCExprLetBinding> bindings, VCExpr body)
+      : base(new List<TypeVariable>(), toSeq(bindings), body)
     {
       Contract.Requires(Cce.NonNullElements(bindings));
       Contract.Requires(body != null);
@@ -2214,7 +2170,6 @@ namespace Microsoft.Boogie.VCExprAST
 
     public override Result Accept<Result, Arg>(IVCExprVisitor<Result, Arg> visitor, Arg arg)
     {
-      //Contract.Requires(visitor != null);
       return visitor.Visit(this, arg);
     }
   }
