@@ -2,6 +2,7 @@ using System;
 using System.Collections.Generic;
 using System.Diagnostics;
 using System.Diagnostics.Contracts;
+using System.Linq;
 using Microsoft.Boogie;
 using Microsoft.Boogie.GraphUtil;
 using VC;
@@ -82,18 +83,9 @@ public class RemoveBackEdges {
 
     #region Create the graph by adding the source node and each edge
 
-    Graph<Block> g = Program.GraphFromImpl(impl);
+    Graph<Block> g = impl.ConvertToReducible(generator.Options);
 
     #endregion
-
-    //Graph<Block> g = program.ProcessLoops(impl);
-
-    g.ComputeLoops(); // this is the call that does all of the processing
-    if (!g.Reducible)
-    {
-      throw new VCGenException("Irreducible flow graphs are unsupported.");
-    }
-
     #endregion
 
     #region Cut the backedges, push assert/assume statements from loop header into predecessors, change them all into assume statements at top of loop, introduce havoc statements
@@ -112,9 +104,12 @@ public class RemoveBackEdges {
 
       List<Cmd> prefixOfPredicateCmdsInit = new List<Cmd>();
       List<Cmd> prefixOfPredicateCmdsMaintained = new List<Cmd>();
-      for (int i = 0, n = header.Cmds.Count; i < n; i++) {
-        if (header.Cmds[i] is not PredicateCmd predicateCmd) {
-          if (header.Cmds[i] is CommentCmd) {
+      for (int i = 0, n = header.Cmds.Count; i < n; i++)
+      {
+        if (header.Cmds[i] is not PredicateCmd predicateCmd)
+        {
+          if (header.Cmds[i] is CommentCmd)
+          {
             // ignore
             continue;
           }
@@ -122,15 +117,19 @@ public class RemoveBackEdges {
           break; // stop when an assignment statement (or any other non-predicate cmd) is encountered
         }
 
-        if (predicateCmd is AssertCmd assertCmd) {
+        if (predicateCmd is AssertCmd assertCmd)
+        {
           AssertCmd initAssertCmd;
 
-          if (generator.Options.ConcurrentHoudini) {
+          if (generator.Options.ConcurrentHoudini)
+          {
             Contract.Assert(taskID >= 0);
             initAssertCmd = generator.Options.Cho[taskID].DisableLoopInvEntryAssert
               ? new LoopInitAssertCmd(assertCmd.tok, Expr.True, assertCmd)
               : new LoopInitAssertCmd(assertCmd.tok, assertCmd.Expr, assertCmd);
-          } else {
+          }
+          else
+          {
             initAssertCmd = new LoopInitAssertCmd(assertCmd.tok, assertCmd.Expr, assertCmd);
           }
 
@@ -143,12 +142,15 @@ public class RemoveBackEdges {
           prefixOfPredicateCmdsInit.Add(initAssertCmd);
 
           LoopInvMaintainedAssertCmd maintainedAssertCmd;
-          if (generator.Options.ConcurrentHoudini) {
+          if (generator.Options.ConcurrentHoudini)
+          {
             Contract.Assert(taskID >= 0);
             maintainedAssertCmd = generator.Options.Cho[taskID].DisableLoopInvMaintainedAssert
               ? new LoopInvMaintainedAssertCmd(assertCmd.tok, Expr.True, assertCmd)
               : new LoopInvMaintainedAssertCmd(assertCmd.tok, assertCmd.Expr, assertCmd);
-          } else {
+          }
+          else
+          {
             maintainedAssertCmd = new LoopInvMaintainedAssertCmd(assertCmd.tok, assertCmd.Expr, assertCmd);
           }
 
@@ -166,9 +168,12 @@ public class RemoveBackEdges {
             id => new TrackedInvariantAssumed(id));
 
           header.Cmds[i] = assume;
-        } else {
+        }
+        else
+        {
           Contract.Assert(predicateCmd is AssumeCmd);
-          if (generator.Options.AlwaysAssumeFreeLoopInvariants) {
+          if (generator.Options.AlwaysAssumeFreeLoopInvariants)
+          {
             // Usually, "free" stuff, like free loop invariants (and the assume statements
             // that stand for such loop invariants) are ignored on the checking side.  This
             // command-line option changes that behavior to always assume the conditions.
@@ -324,16 +329,10 @@ public class RemoveBackEdges {
 
       #region Create the graph by adding the source node and each edge
 
-      Graph<Block> g = Program.GraphFromImpl(impl);
+      Graph<Block> g = impl.ConvertToReducible(generator.Options);
 
       #endregion
 
-      g.ComputeLoops(); // this is the call that does all of the processing
-      if (!g.Reducible)
-      {
-        throw new VCGenException("Irreducible flow graphs are unsupported.");
-      }
-
       #endregion
 
       foreach (Block header in Cce.NonNull(g.Headers))
